// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.io.wizard;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.net.URI;

import org.apache.log4j.Logger;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IWorkbench;

import eu.etaxonomy.cdm.database.DbSchemaValidation;
import eu.etaxonomy.cdm.io.specimen.abcd206.in.Abcd206ImportConfigurator;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.internal.TaxeditorStorePlugin;

/**
 * <p>AbcdImportWizard class.</p>
 *
 * @author n.hoffmann
 * @created Jun 16, 2010
 * @version 1.0
 */
public class AbcdImportWizard extends AbstractImportWizard<Abcd206ImportConfigurator> {

    private static final Logger logger = Logger.getLogger(AbcdImportWizard.class);

	private Abcd206ImportConfigurator configurator;
	private ImportFromFileDataSourceWizardPage dataSourcePage;
	private ClassificationChooserWizardPage classificationChooserWizardPage;

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.io.wizard.AbstractImportWizard#getConfigurator()
	 */
	/** {@inheritDoc} */
	@Override
	public Abcd206ImportConfigurator getConfigurator() {
		return configurator;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.Wizard#performFinish()
	 */
	/** {@inheritDoc} */
	@Override
	public boolean performFinish() {
		URI source = dataSourcePage.getUri();
		try {
            configurator.setSource(new FileInputStream(new File(source)));
        } catch (FileNotFoundException e) {
            MessagingUtils.errorDialog("File not found.", this, "Import file was not found.", TaxeditorStorePlugin.PLUGIN_ID, e, false);
            logger.error("File not found!", e);
            return false;
        }
		configurator.setDbSchemaValidation(DbSchemaValidation.CREATE);

		if(classificationChooserWizardPage.getClassification()!=null){
		    configurator.setClassificationUuid(classificationChooserWizardPage.getClassification().getUuid());
		}
		configurator.setReportUri(classificationChooserWizardPage.getReportUri());

		CdmStore.getImportManager().run(configurator);
		return true;

	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchWizard#init(org.eclipse.ui.IWorkbench, org.eclipse.jface.viewers.IStructuredSelection)
	 */
	/** {@inheritDoc} */
	@Override
    public void init(IWorkbench workbench, IStructuredSelection selection) {
		super.init(workbench, selection);
		configurator =  CdmStore.getImportManager().AbcdConfigurator();
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.io.AbstractImportWizard#addPages()
	 */
	/** {@inheritDoc} */
	@Override
	public void addPages() {
		super.addPages();

		classificationChooserWizardPage = ClassificationChooserWizardPage.createPage();
		dataSourcePage = ImportFromFileDataSourceWizardPage.XML();
		addPage(classificationChooserWizardPage);
		addPage(dataSourcePage);
	}
}
