// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.model;

import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.agent.Person;
import eu.etaxonomy.cdm.model.agent.Team;
import eu.etaxonomy.cdm.model.agent.TeamOrPersonBase;

/**
 * Utility class for handling of authorship related elements.
 *
 * @author n.hoffmann
 * @created Sep 8, 2010
 * @version 1.0
 */
public class AuthorHelper {

    /**
     * If the given {@link TeamOrPersonBase} is a {@link Person} this methods
     * creates a new {@link Team} with this person. If it already a team the
     * team is returned.
     * 
     * @param teamOrPerson
     *            a team or a person
     * @return the given team or a newly created team with the given person
     */
	public static Team getAuthor(TeamOrPersonBase<?> teamOrPerson){
		if(teamOrPerson == null){
			return null;
		}
		Object deproxiedObject = HibernateProxyHelper.deproxy(teamOrPerson);
		if(deproxiedObject instanceof Person){
			Person person = (Person) deproxiedObject;
			Team team = Team.NewInstance();
			team.addTeamMember(person);
			return team;
		}
		else if(deproxiedObject instanceof Team){
			return (Team) deproxiedObject;
		}else{
			throw new IllegalArgumentException("Given object os of the wrong type");
		}
	}
}
