// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.model;

import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.widgets.Control;

/**
 * <p>TextHelper class.</p>
 *
 * @author n.hoffmann
 * @created Mar 30, 2010
 * @version 1.0
 */
public class TextHelper {
	
	/** Constant <code>ELLIPSIS="..."</code> */
	public static final String ELLIPSIS = "...";
		
	/**
	 * <p>shortenText</p>
	 *
	 * @param textValue a {@link java.lang.String} object.
	 * @param control a {@link org.eclipse.swt.widgets.Control} object.
	 * @see org.eclipse.jface.dialogs.Dialog#shortenText(String, Control)
	 * @return a {@link java.lang.String} object.
	 */
	public static String shortenText(String textValue, Control control) {
		if (textValue == null) {
			return null;
		}
		GC gc = new GC(control);
		int maxWidth = control.getBounds().width;
		int maxExtent = gc.textExtent(textValue).x;
		if (maxExtent < maxWidth) {
			gc.dispose();
			return textValue;
		}
		int length = textValue.length();
		int charsToClip = Math.round(0.95f*length * (1 - ((float)maxWidth/maxExtent)));
		
		int end = length - charsToClip;
		while (end > 0) {
			String s1 = textValue.substring(0, end);
			String s = s1 + ELLIPSIS;
			int l = gc.textExtent(s).x;
			if (l < maxWidth) {
				gc.dispose();
				return s;
			}
			end--;
		}
		gc.dispose();
		return textValue;
		
	}
	
	public static String deproxyClassName(Class clazz){
		String name = clazz.getSimpleName();
		if (name.indexOf("_$$")>-1){
			return name.substring(0, name.indexOf("_$$"));
		}else{
			return name;
		}
	}
}
