// $Id$
/**
 * Copyright (C) 2014 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.ui.dialog.selection;

import java.util.UUID;

import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.service.molecular.IAmplificationService;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.molecular.Amplification;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.newWizard.AbstractNewEntityWizard;
import eu.etaxonomy.taxeditor.newWizard.NewAmplificationWizard;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author pplitzner
 * @date 11.03.2014
 *
 */
public class AmplificationSelectionDialog extends
AbstractFilteredCdmResourceSelectionDialog<Amplification> {

    public static Amplification select(Shell shell, ConversationHolder conversation, Amplification amplification){
        AmplificationSelectionDialog dialog = new AmplificationSelectionDialog(shell, conversation,
                "Choose Amplification", false, AmplificationSelectionDialog.class.getCanonicalName(), amplification);
        return getSelectionFromDialog(dialog);
    }

    /**
     * <p>Constructor for FilteredDerivedUnitSelectionDialog.</p>
     */
    protected AmplificationSelectionDialog(Shell shell, ConversationHolder conversation, String title,
            boolean multi, String settings, Amplification cdmObject) {
        super(shell, conversation, title, multi, settings, cdmObject);
    }

    /** {@inheritDoc} */
    @Override
    protected Amplification getPersistentObject(UUID uuid) {
        Object object = CdmStore.getService(IAmplificationService.class).load(uuid);

        Amplification amplification = (Amplification) HibernateProxyHelper.deproxy(object);

        if(amplification != null){
            return amplification;
        }
        MessagingUtils.error(this.getClass(), "Selected element is not an amplification", null);
        return null;
    }

    /** {@inheritDoc} */
    @Override
    protected void initModel() {
        model = CdmStore.getService(IAmplificationService.class).getAmplificationUuidAndLabelCache();
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.ui.dialog.selection.AbstractFilteredCdmResourceSelectionDialog#getTitle(eu.etaxonomy.cdm.model.common.ICdmBase)
     */
    @Override
    protected String getTitle(Amplification cdmObject) {
        if(cdmObject!=null){
            return cdmObject.getLabelCache();
        }
        return super.getTitle(cdmObject);
    }

    /** {@inheritDoc} */
    @Override
    protected AbstractNewEntityWizard getNewEntityWizard(String parameter) {
        return new NewAmplificationWizard();
    }

    /** {@inheritDoc} */
    @Override
    protected String getNewWizardLinkText() {
        return String.format("Create a new <a>%1s</a>", "Amplification ");
    }
}
