/**
 *
 */
package eu.etaxonomy.taxeditor.ui.element;

import java.net.URI;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.events.TypedEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.forms.IFormColors;
import org.eclipse.ui.forms.widgets.ExpandableComposite;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;
import org.eclipse.ui.internal.forms.widgets.FormFonts;
import org.joda.time.DateTime;
import org.joda.time.Partial;
import org.springframework.security.core.GrantedAuthority;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.facade.DerivedUnitFacade;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.agent.Person;
import eu.etaxonomy.cdm.model.agent.Team;
import eu.etaxonomy.cdm.model.common.Annotation;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.common.Credit;
import eu.etaxonomy.cdm.model.common.DefinedTerm;
import eu.etaxonomy.cdm.model.common.DefinedTermBase;
import eu.etaxonomy.cdm.model.common.Extension;
import eu.etaxonomy.cdm.model.common.GrantedAuthorityImpl;
import eu.etaxonomy.cdm.model.common.Group;
import eu.etaxonomy.cdm.model.common.ICdmBase;
import eu.etaxonomy.cdm.model.common.IEnumTerm;
import eu.etaxonomy.cdm.model.common.IdentifiableSource;
import eu.etaxonomy.cdm.model.common.Identifier;
import eu.etaxonomy.cdm.model.common.LSID;
import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.common.LanguageString;
import eu.etaxonomy.cdm.model.common.Marker;
import eu.etaxonomy.cdm.model.common.TermType;
import eu.etaxonomy.cdm.model.common.TermVocabulary;
import eu.etaxonomy.cdm.model.common.TimePeriod;
import eu.etaxonomy.cdm.model.common.User;
import eu.etaxonomy.cdm.model.common.VersionableEntity;
import eu.etaxonomy.cdm.model.description.CategoricalData;
import eu.etaxonomy.cdm.model.description.CommonTaxonName;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.DescriptionElementSource;
import eu.etaxonomy.cdm.model.description.Distribution;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.IndividualsAssociation;
import eu.etaxonomy.cdm.model.description.KeyStatement;
import eu.etaxonomy.cdm.model.description.MeasurementUnit;
import eu.etaxonomy.cdm.model.description.QuantitativeData;
import eu.etaxonomy.cdm.model.description.State;
import eu.etaxonomy.cdm.model.description.StateData;
import eu.etaxonomy.cdm.model.description.StatisticalMeasure;
import eu.etaxonomy.cdm.model.description.StatisticalMeasurementValue;
import eu.etaxonomy.cdm.model.description.TaxonInteraction;
import eu.etaxonomy.cdm.model.description.TextData;
import eu.etaxonomy.cdm.model.location.NamedArea;
import eu.etaxonomy.cdm.model.location.NamedAreaLevel;
import eu.etaxonomy.cdm.model.location.Point;
import eu.etaxonomy.cdm.model.media.ImageFile;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.cdm.model.media.MediaRepresentation;
import eu.etaxonomy.cdm.model.media.MediaRepresentationPart;
import eu.etaxonomy.cdm.model.media.Rights;
import eu.etaxonomy.cdm.model.molecular.DnaSample;
import eu.etaxonomy.cdm.model.name.NameRelationship;
import eu.etaxonomy.cdm.model.name.NameTypeDesignation;
import eu.etaxonomy.cdm.model.name.NomenclaturalStatus;
import eu.etaxonomy.cdm.model.name.SpecimenTypeDesignation;
import eu.etaxonomy.cdm.model.occurrence.DerivedUnit;
import eu.etaxonomy.cdm.model.occurrence.DeterminationEvent;
import eu.etaxonomy.cdm.model.reference.Reference;
import eu.etaxonomy.cdm.model.taxon.SynonymRelationship;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.cdm.strategy.parser.ParserProblem;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.ui.combo.EnumComboElement;
import eu.etaxonomy.taxeditor.ui.combo.TermComboElement;
import eu.etaxonomy.taxeditor.ui.combo.VocabularyComboElement;
import eu.etaxonomy.taxeditor.ui.element.MinMaxTextSection.UnitType;
import eu.etaxonomy.taxeditor.ui.mvc.element.DateElement;
import eu.etaxonomy.taxeditor.ui.openurl.IOpenUrlEnabled;
import eu.etaxonomy.taxeditor.ui.openurl.OpenUrlSelectorElement;
import eu.etaxonomy.taxeditor.ui.password.EditPasswordElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailSection;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionElement;
import eu.etaxonomy.taxeditor.ui.section.EmptyElement;
import eu.etaxonomy.taxeditor.ui.section.agent.InstitutionDetailElement;
import eu.etaxonomy.taxeditor.ui.section.agent.InstitutionDetailSection;
import eu.etaxonomy.taxeditor.ui.section.agent.PersonDetailElement;
import eu.etaxonomy.taxeditor.ui.section.agent.PersonDetailSection;
import eu.etaxonomy.taxeditor.ui.section.agent.TeamDetailElement;
import eu.etaxonomy.taxeditor.ui.section.agent.TeamDetailSection;
import eu.etaxonomy.taxeditor.ui.section.agent.TeamMemberElement;
import eu.etaxonomy.taxeditor.ui.section.agent.TeamMemberSection;
import eu.etaxonomy.taxeditor.ui.section.agent.TeamOrPersonBaseDetailSection;
import eu.etaxonomy.taxeditor.ui.section.classification.ClassificationDetailElement;
import eu.etaxonomy.taxeditor.ui.section.classification.ClassificationDetailSection;
import eu.etaxonomy.taxeditor.ui.section.classification.TaxonNodeDetailElement;
import eu.etaxonomy.taxeditor.ui.section.classification.TaxonNodeDetailSection;
import eu.etaxonomy.taxeditor.ui.section.common.ReferenceEntityDetailElement;
import eu.etaxonomy.taxeditor.ui.section.common.ReferencedEntityDetailSection;
import eu.etaxonomy.taxeditor.ui.section.description.DerivedUnitElement;
import eu.etaxonomy.taxeditor.ui.section.description.DescribedSpecimenSection;
import eu.etaxonomy.taxeditor.ui.section.description.DescriptionDetailElement;
import eu.etaxonomy.taxeditor.ui.section.description.DescriptionDetailSection;
import eu.etaxonomy.taxeditor.ui.section.description.DescriptionElementDetailElement;
import eu.etaxonomy.taxeditor.ui.section.description.DescriptionElementDetailSection;
import eu.etaxonomy.taxeditor.ui.section.description.DescriptionElementMediaSection;
import eu.etaxonomy.taxeditor.ui.section.description.DescriptionElementSourceElement;
import eu.etaxonomy.taxeditor.ui.section.description.DescriptionElementSourceSection;
import eu.etaxonomy.taxeditor.ui.section.description.DescriptionSourceElement;
import eu.etaxonomy.taxeditor.ui.section.description.DescriptionSourceSection;
import eu.etaxonomy.taxeditor.ui.section.description.MediaDetailsSection;
import eu.etaxonomy.taxeditor.ui.section.description.ModifierElement;
import eu.etaxonomy.taxeditor.ui.section.description.ModifierSection;
import eu.etaxonomy.taxeditor.ui.section.description.NaturalLanguageDetailElement;
import eu.etaxonomy.taxeditor.ui.section.description.NaturalLanguageSection;
import eu.etaxonomy.taxeditor.ui.section.description.ScopeElement;
import eu.etaxonomy.taxeditor.ui.section.description.ScopeSection;
import eu.etaxonomy.taxeditor.ui.section.description.StateDataElement;
import eu.etaxonomy.taxeditor.ui.section.description.StateDataSection;
import eu.etaxonomy.taxeditor.ui.section.description.StatisticalMeasurementValueElement;
import eu.etaxonomy.taxeditor.ui.section.description.StatisticalMeasurementValueSection;
import eu.etaxonomy.taxeditor.ui.section.description.detail.AbstractDetailedDescriptionDetailElement;
import eu.etaxonomy.taxeditor.ui.section.description.detail.CategoricalDataDetailElement;
import eu.etaxonomy.taxeditor.ui.section.description.detail.CommonNameDetailElement;
import eu.etaxonomy.taxeditor.ui.section.description.detail.DistributionDetailElement;
import eu.etaxonomy.taxeditor.ui.section.description.detail.IndividualsAssociationDetailElement;
import eu.etaxonomy.taxeditor.ui.section.description.detail.QuantitativeDataDetailElement;
import eu.etaxonomy.taxeditor.ui.section.description.detail.TaxonInteractionDetailElement;
import eu.etaxonomy.taxeditor.ui.section.description.detail.TextDataDetailElement;
import eu.etaxonomy.taxeditor.ui.section.feature.FeatureDistributionDetailElement;
import eu.etaxonomy.taxeditor.ui.section.feature.FeatureDistributionDetailSection;
import eu.etaxonomy.taxeditor.ui.section.grantedAuthority.GrantedAuthorityCollectionElement;
import eu.etaxonomy.taxeditor.ui.section.grantedAuthority.GrantedAuthorityDetailElement;
import eu.etaxonomy.taxeditor.ui.section.grantedAuthority.GrantedAuthorityDetailSection;
import eu.etaxonomy.taxeditor.ui.section.group.GroupDetailElement;
import eu.etaxonomy.taxeditor.ui.section.group.GroupDetailSection;
import eu.etaxonomy.taxeditor.ui.section.group.MemberDetailElement;
import eu.etaxonomy.taxeditor.ui.section.group.MemberDetailSection;
import eu.etaxonomy.taxeditor.ui.section.identifier.IdentifierDetailElement;
import eu.etaxonomy.taxeditor.ui.section.identifier.IdentifierDetailSection;
import eu.etaxonomy.taxeditor.ui.section.key.PolytomousKeyDetailElement;
import eu.etaxonomy.taxeditor.ui.section.key.PolytomousKeyDetailSection;
import eu.etaxonomy.taxeditor.ui.section.key.PolytomousKeyNodeDetailElement;
import eu.etaxonomy.taxeditor.ui.section.key.PolytomousKeyNodeDetailSection;
import eu.etaxonomy.taxeditor.ui.section.key.ScopeRestrictionSection;
import eu.etaxonomy.taxeditor.ui.section.key.TaxonomicScopeSection;
import eu.etaxonomy.taxeditor.ui.section.media.ImageFileElement;
import eu.etaxonomy.taxeditor.ui.section.media.MediaDetailElement;
import eu.etaxonomy.taxeditor.ui.section.media.MediaMetaElement;
import eu.etaxonomy.taxeditor.ui.section.media.MediaRepresentationElement;
import eu.etaxonomy.taxeditor.ui.section.media.MediaRepresentationPartElement;
import eu.etaxonomy.taxeditor.ui.section.media.MediaRepresentationPartSection;
import eu.etaxonomy.taxeditor.ui.section.media.MediaRepresentationSection;
import eu.etaxonomy.taxeditor.ui.section.media.MediaSection;
import eu.etaxonomy.taxeditor.ui.section.name.AuthorshipDetailElement;
import eu.etaxonomy.taxeditor.ui.section.name.AuthorshipDetailSection;
import eu.etaxonomy.taxeditor.ui.section.name.HybridDetailElement;
import eu.etaxonomy.taxeditor.ui.section.name.HybridDetailSection;
import eu.etaxonomy.taxeditor.ui.section.name.NameDetailElement;
import eu.etaxonomy.taxeditor.ui.section.name.NameDetailSection;
import eu.etaxonomy.taxeditor.ui.section.name.NameRelationshipDetailElement;
import eu.etaxonomy.taxeditor.ui.section.name.NameRelationshipDetailSection;
import eu.etaxonomy.taxeditor.ui.section.name.NameTypeDesignationElement;
import eu.etaxonomy.taxeditor.ui.section.name.NomenclaturalStatusElement;
import eu.etaxonomy.taxeditor.ui.section.name.NomenclaturalStatusSection;
import eu.etaxonomy.taxeditor.ui.section.name.NonViralNameDetailElement;
import eu.etaxonomy.taxeditor.ui.section.name.NonViralNameDetailSection;
import eu.etaxonomy.taxeditor.ui.section.name.ProtologueElement;
import eu.etaxonomy.taxeditor.ui.section.name.ProtologueSection;
import eu.etaxonomy.taxeditor.ui.section.name.SpecimenTypeDesignationElement;
import eu.etaxonomy.taxeditor.ui.section.name.SynonymRelationshipDetailElement;
import eu.etaxonomy.taxeditor.ui.section.name.SynonymRelationshipDetailSection;
import eu.etaxonomy.taxeditor.ui.section.name.TypeDesignationSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.CollectingAreasDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.CollectionDetailElement;
import eu.etaxonomy.taxeditor.ui.section.occurrence.CurrentDeterminationDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.DerivedUnitBaseDetailElement;
import eu.etaxonomy.taxeditor.ui.section.occurrence.DerivedUnitBaseDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.DerivedUnitGeneralDetailElement;
import eu.etaxonomy.taxeditor.ui.section.occurrence.DerivedUnitGeneralDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.DeterminationDetailElement;
import eu.etaxonomy.taxeditor.ui.section.occurrence.DeterminationDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.DeterminationEventDetailElement;
import eu.etaxonomy.taxeditor.ui.section.occurrence.DeterminationHistoryDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.EmptySection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.FieldUnitDetailElement;
import eu.etaxonomy.taxeditor.ui.section.occurrence.FieldUnitDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.FieldUnitGeneralDetailElement;
import eu.etaxonomy.taxeditor.ui.section.occurrence.FieldUnitGeneralDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.GatheringEventDetailElement;
import eu.etaxonomy.taxeditor.ui.section.occurrence.GatheringEventDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.GeoScopeDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.NamedAreaDetailElement;
import eu.etaxonomy.taxeditor.ui.section.occurrence.OriginalLabelDataSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.SourceCollectionDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.SpecimenCollectionDetailElement;
import eu.etaxonomy.taxeditor.ui.section.occurrence.SpecimenCollectionDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.SpecimenHierarchyDetailElement;
import eu.etaxonomy.taxeditor.ui.section.occurrence.SpecimenHierarchyDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.association.TaxonAssociationDetailElement;
import eu.etaxonomy.taxeditor.ui.section.occurrence.association.TaxonAssociationDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.derivedUnit.PreservedSpecimenCurrentDeterminationDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.derivedUnit.PreservedSpecimenDetailElement;
import eu.etaxonomy.taxeditor.ui.section.occurrence.derivedUnit.PreservedSpecimenDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.derivedUnit.PreservedSpecimenDeterminationDetailElement;
import eu.etaxonomy.taxeditor.ui.section.occurrence.derivedUnit.PreservedSpecimenDeterminationDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.derivedUnit.PreservedSpecimenDeterminationHistoryDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.derivedUnit.PreservedSpecimenGeneralDetailElement;
import eu.etaxonomy.taxeditor.ui.section.occurrence.derivedUnit.PreservedSpecimenGeneralDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.derivedUnit.PreservedSpecimenSourceCollectionDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.AbstractSampleDesignationDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.AmplificationCloningDetailElement;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.AmplificationCloningDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.AmplificationGelPhotoCollectionDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.AmplificationGelPhotoDetailElement;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.AmplificationGeneralDetailElement;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.AmplificationGeneralDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.AmplificationPrimerDetailElement;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.AmplificationPrimerDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.CurrentSampleDesignationDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.DnaQualityDetailElement;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.DnaQualityDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.DnaSampleGeneralDetailElement;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.DnaSampleGeneralDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.DnaSamplePreparationPreservationDetailElement;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.DnaSamplePreparationPreservationSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.PrimerGeneralDetailElement;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.SampleDesignationDetailElement;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.SampleDesignationDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.SampleDesignationHistoryDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.SampleDesignationTextDetailElement;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.SequenceContigFileCollectionDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.SequenceGeneralDetailElement;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.SequenceGeneralDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.SequenceReferenceCollectionDetailElement;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.SequenceReferenceCollectionDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.SingleReadGeneralDetailElement;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.SingleReadGeneralDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.SingleReadPherogramCollectionDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.TissueSampleGeneralDetailElement;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.TissueSampleGeneralDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.media.MediaSpecimenGeneralDetailElement;
import eu.etaxonomy.taxeditor.ui.section.occurrence.media.MediaSpecimenGeneralDetailSection;
import eu.etaxonomy.taxeditor.ui.section.reference.NomenclaturalReferenceDetailElement;
import eu.etaxonomy.taxeditor.ui.section.reference.NomenclaturalReferenceDetailSection;
import eu.etaxonomy.taxeditor.ui.section.reference.ReferenceDetailElement;
import eu.etaxonomy.taxeditor.ui.section.reference.ReferenceDetailSection;
import eu.etaxonomy.taxeditor.ui.section.supplemental.AnnotationElement;
import eu.etaxonomy.taxeditor.ui.section.supplemental.AnnotationSection;
import eu.etaxonomy.taxeditor.ui.section.supplemental.CdmBaseElement;
import eu.etaxonomy.taxeditor.ui.section.supplemental.CdmBaseSection;
import eu.etaxonomy.taxeditor.ui.section.supplemental.CreditElement;
import eu.etaxonomy.taxeditor.ui.section.supplemental.CreditSection;
import eu.etaxonomy.taxeditor.ui.section.supplemental.ExtensionElement;
import eu.etaxonomy.taxeditor.ui.section.supplemental.ExtensionSection;
import eu.etaxonomy.taxeditor.ui.section.supplemental.HeadlineSection;
import eu.etaxonomy.taxeditor.ui.section.supplemental.IdentifiableSourceElement;
import eu.etaxonomy.taxeditor.ui.section.supplemental.MarkerElement;
import eu.etaxonomy.taxeditor.ui.section.supplemental.MarkerSection;
import eu.etaxonomy.taxeditor.ui.section.supplemental.RightsElement;
import eu.etaxonomy.taxeditor.ui.section.supplemental.RightsSection;
import eu.etaxonomy.taxeditor.ui.section.supplemental.SourceSection;
import eu.etaxonomy.taxeditor.ui.section.supplemental.VersionElement;
import eu.etaxonomy.taxeditor.ui.section.supplemental.VersionSection;
import eu.etaxonomy.taxeditor.ui.section.taxon.ParsingMessageElement;
import eu.etaxonomy.taxeditor.ui.section.taxon.ParsingMessagesSection;
import eu.etaxonomy.taxeditor.ui.section.taxon.TaxonBaseDetailElement;
import eu.etaxonomy.taxeditor.ui.section.taxon.TaxonBaseDetailSection;
import eu.etaxonomy.taxeditor.ui.section.taxon.TaxonDetailElement;
import eu.etaxonomy.taxeditor.ui.section.taxon.TaxonRelationshipDetailElement;
import eu.etaxonomy.taxeditor.ui.section.taxon.TaxonRelationshipDetailSection;
import eu.etaxonomy.taxeditor.ui.section.user.GroupsByUserDetailElement;
import eu.etaxonomy.taxeditor.ui.section.user.GroupsByUserDetailSection;
import eu.etaxonomy.taxeditor.ui.section.user.UserDetailElement;
import eu.etaxonomy.taxeditor.ui.section.user.UserDetailSection;
import eu.etaxonomy.taxeditor.ui.section.userecords.UseRecordDetailElement;
import eu.etaxonomy.taxeditor.ui.section.userecords.UseRecordDetailSection;
import eu.etaxonomy.taxeditor.ui.section.vocabulary.DefinedTermDetailElement;
import eu.etaxonomy.taxeditor.ui.section.vocabulary.DefinedTermDetailSection;
import eu.etaxonomy.taxeditor.ui.section.vocabulary.FeatureDetailElement;
import eu.etaxonomy.taxeditor.ui.section.vocabulary.MeasurementUnitCollectionElement;
import eu.etaxonomy.taxeditor.ui.section.vocabulary.MeasurementUnitCollectionSection;
import eu.etaxonomy.taxeditor.ui.section.vocabulary.NamedAreaDetailSection;
import eu.etaxonomy.taxeditor.ui.section.vocabulary.RecommendedModifierVocabulariesCollectionSection;
import eu.etaxonomy.taxeditor.ui.section.vocabulary.RecommendedModifierVocabulariesElement;
import eu.etaxonomy.taxeditor.ui.section.vocabulary.StateVocabularyCollectionElement;
import eu.etaxonomy.taxeditor.ui.section.vocabulary.StateVocabularyCollectionSection;
import eu.etaxonomy.taxeditor.ui.section.vocabulary.StatisticalMeasureCollectionElement;
import eu.etaxonomy.taxeditor.ui.section.vocabulary.StatisticalMeasureCollectionSection;
import eu.etaxonomy.taxeditor.ui.section.vocabulary.TermVocabularyDetailElement;
import eu.etaxonomy.taxeditor.ui.section.vocabulary.TermVocabularyDetailSection;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElementWithAbbreviatedTitle;
import eu.etaxonomy.taxeditor.ui.selection.NomenclaturalAuthorTeamSelectionElement;
import eu.etaxonomy.taxeditor.ui.selection.TaxonNodeSelectionElement;

/**
 * <p>
 * CdmFormFactory class.
 * </p>
 *
 * @author n.hoffmann
 * @created Feb 24, 2010
 * @version 1.0
 */
public class CdmFormFactory extends FormToolkit {

    private BoldFontHolder2 boldFontHolder2;
    private MouseListener selectionMouseHandler;
    private FocusListener selectionFocusHandler;

    private final Set<SelectionListener> selectionListenerList = new HashSet<SelectionListener>();

    private final List<IPropertyChangeListener> propertyChangeListeners = new ArrayList<IPropertyChangeListener>();

    private final int orientation = Window.getDefaultOrientation();
    private ISelectionProvider selectionProvider;

    /** Constant <code>EMPTY_SELECTION</code> */
    public static ISelection EMPTY_SELECTION = new ISelection() {
        @Override
        public boolean isEmpty() {
            return true;
        }
    };

    /**
     *
     * @author n.hoffmann
     * @date Jan 25, 2010
     *
     */
    private class SelectionMouseHandler extends MouseAdapter {
        @Override
        public void mouseDown(MouseEvent e) {
            notifySelectionListeners(e);
        }
    }

    /**
     *
     * @author n.hoffmann
     * @date Jan 25, 2010
     *
     */
    private class SelectionFocusHandler extends FocusAdapter {
        @Override
        public void focusGained(FocusEvent e) {
            notifySelectionListeners(e);
        }
    }

    private void notifySelectionListeners(TypedEvent e) {
        Event event = new Event();
        event.widget = e.widget;
        SelectionEvent selectionEvent = new SelectionEvent(event);

        for (SelectionListener listener : selectionListenerList) {
            listener.widgetSelected(selectionEvent);
        }
    }

    /**
     * <p>
     * Constructor for CdmFormFactory.
     * </p>
     *
     * @param display
     *            a {@link org.eclipse.swt.widgets.Display} object.
     * @param selectionProvider
     *            a {@link org.eclipse.jface.viewers.ISelectionProvider} object.
     */
    public CdmFormFactory(Display display, ISelectionProvider selectionProvider) {
        super(display);
        this.selectionProvider = selectionProvider;
        init();
    }

    /**
     * <p>
     * Constructor for CdmFormFactory.
     * </p>
     *
     * @param display
     *            a {@link org.eclipse.swt.widgets.Display} object.
     */
    public CdmFormFactory(Display display) {
        super(display);
        init();
    }

    /**
	 *
	 */
    private void init() {
        boldFontHolder2 = new BoldFontHolder2();
        selectionMouseHandler = new SelectionMouseHandler();
        selectionFocusHandler = new SelectionFocusHandler();
    }

    /**
     * Creates an instance initialized with the correct selectionProvider
     *
     * Make sure to remove the instance when the entityComposite disposes via
     * destroySelectionArbitrator(..)
     *
     * @param entityElement
     *            a {@link eu.etaxonomy.taxeditor.ui.element.IEntityElement}
     *            object.
     * @return a {@link eu.etaxonomy.taxeditor.ui.element.SelectionArbitrator}
     *         object.
     */
    public SelectionArbitrator createSelectionArbitrator(IEntityElement entityElement) {
        SelectionArbitrator selectionArbitrator = new SelectionArbitrator(entityElement);
        selectionArbitrator.addSelectionProvider(selectionProvider);
        selectionProvider.addSelectionChangedListener(selectionArbitrator);
        addSelectionListener(selectionArbitrator);
        return selectionArbitrator;
    }

    /**
     * <p>
     * destroySelectionArbitrator
     * </p>
     *
     * @param selectionArbitrator
     *            a
     *            {@link eu.etaxonomy.taxeditor.ui.element.SelectionArbitrator}
     *            object.
     */
    public void destroySelectionArbitrator(SelectionArbitrator selectionArbitrator) {
        removeSelectionListener(selectionArbitrator);
        if (selectionProvider != null) {
            selectionProvider.removeSelectionChangedListener(selectionArbitrator);
        } else {
            MessagingUtils.error(this.getClass(),
                    "Tried to destroy a selection listener from this factories listeners but was null", null);
        }
    }

    /**
     * <p>
     * Adapts the {@link AbstractCdmFormElement}:<br>
     * - sets the {@link IPropertyChangeListener}s handled by this class
     * </p>
     *
     * @param formElement
     *            a
     *            {@link eu.etaxonomy.taxeditor.ui.element.AbstractCdmFormElement}
     *            object.
     */
    public void adapt(ICdmFormElement formElement) {
        formElement.setPropertyChangeListeners(propertyChangeListeners);
    }

    /** {@inheritDoc} */
    @Override
    public void adapt(Control control, boolean trackFocus, boolean trackKeyboard) {
        if (trackFocus) {
            control.addFocusListener(selectionFocusHandler);
        }
        super.adapt(control, trackFocus, trackKeyboard);
    }

    /** {@inheritDoc} */
    @Override
    public void adapt(Composite composite) {
        composite.addMouseListener(selectionMouseHandler);
        super.adapt(composite);
    }

    /**
     * <p>
     * destroyElement
     * </p>
     *
     * @param formElement
     *            a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement}
     *            object.
     */
    public void destroyElement(ICdmFormElement formElement) {
        // return if element was not initialized
        if (formElement == null) {
            return;
        }
        // destroy selection arbitrator, if any
        if (formElement instanceof ISelectableElement) {
            destroySelectionArbitrator(((ISelectableElement) formElement).getSelectionArbitrator());
        }
        // remove this element form its parents list of elements
        // ICdmFormElement parentElement = formElement.getParentElement();
        // if(parentElement != null){
        // parentElement.removeElement(formElement);
        // }
        // call destroy on child elements recursively
        for (ICdmFormElement childElement : formElement.getElements()) {
            destroyElement(childElement);
        }
        // dispose of the controls
        for (Control control : formElement.getControls()) {
            // we added the layoutComposite of the parental element as the
            // layout composite to this formElement
            // but we do not want to destroy it.
            if (control.equals(formElement.getLayoutComposite())) {
                continue;
            } else {
                control.dispose();
                control = null;
            }
        }
    }

    /**
     * <p>
     * createEmptyCell
     * </p>
     *
     * @param parent
     *            a {@link org.eclipse.swt.widgets.Composite} object.
     * @return a {@link org.eclipse.swt.widgets.Label} object.
     */
    public Label createEmptyCell(Composite parent) {
        return this.createLabel(parent, null);
    }

    /**
     * <p>
     * createMultiLanguageTextElement
     * </p>
     *
     * @param parentElement
     *            a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement}
     *            object.
     * @param labelString
     *            a {@link java.lang.String} object.
     * @param multilanguageText
     *            a {@link java.util.Map} object.
     * @param textHeight
     *            a int.
     * @param style
     *            a int.
     * @return a
     *         {@link eu.etaxonomy.taxeditor.ui.element.MultilanguageTextElement}
     *         object.
     */
    public MultilanguageTextElement createMultiLanguageTextElement(ICdmFormElement parentElement, String labelString,
            Map<Language, LanguageString> multilanguageText, int textHeight, int style) {
        MultilanguageTextElement element = new MultilanguageTextElement(this, parentElement, labelString,
                multilanguageText, textHeight, style);
        adapt(element);
        parentElement.addElement(element);
        return element;
    }

    public KeyStatementElement createKeyStatementElement(ICdmFormElement parentElement, String labelString,
            KeyStatement keyStatement, int textHeight, int style) {
        KeyStatementElement element = new KeyStatementElement(this, parentElement, labelString, keyStatement,
                textHeight, style);
        adapt(element);
        parentElement.addElement(element);
        return element;
    }

    /**
     * Creates a text field with a label. The initial content will be the return value of the initalObject's {@link #toString()}
     * method.<br>
     * <b>Note</b>: if initialObject is <code>null</code> then an empty string is used.
     * @param parentElement the parent container
     * @param labelString the label name
     * @param initialObject the object from which the <code>toString()</code> method is called
     * @param style {@link SWT} style constant
     * @return the created textfield with label
     */
    public TextWithLabelElement createTextWithLabelElement(ICdmFormElement parentElement, String labelString,
            Object initialObject, int style) {
        return createTextWithLabelElement(parentElement, labelString, initialObject==null?"":initialObject.toString(), style);
    }

    /**
     * <p>
     * createTextWithLabelElement
     * </p>
     *
     * @param parentElement
     *            a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement}
     *            object.
     * @param labelString
     *            a {@link java.lang.String} object.
     * @param initialText
     *            a {@link java.lang.String} object.
     * @param style
     *            a int.
     * @return a {@link eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement}
     *         object.
     */
    public TextWithLabelElement createTextWithLabelElement(ICdmFormElement parentElement, String labelString,
            String initialText, int style) {
        return createTextWithLabelElement(parentElement, labelString, initialText, null, style);
    }
    /**
     * <p>
     * createTextWithLabelElement
     * </p>
     *
     * @param parentElement
     *            a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement}
     *            object.
     * @param labelString
     *            a {@link java.lang.String} object.
     * @param initialText
     *            a {@link java.lang.String} object.
     * @param textLimit maximal number of characters allowed
     * @param style
     *            a int.
     * @return a {@link eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement}
     *         object.
     */
    public TextWithLabelElement createTextWithLabelElement(ICdmFormElement parentElement, String labelString,
            String initialText, Integer textLimit, int style) {
        if(initialText==null){
            initialText = "";
        }

        TextWithLabelElement element = new TextWithLabelElement(this, parentElement, labelString, initialText, null,
                textLimit, style);
        adapt(element);
        parentElement.addElement(element);
        return element;
    }

    public TextWithLabelElement createMultiLineTextWithLabel(ICdmFormElement parentElement, String labelString,
            int textHeight, int style) {
        TextWithLabelElement element = new TextWithLabelElement(this, parentElement, labelString, "", textHeight, true, style);
        adapt(element);
        parentElement.addElement(element);
        return element;
    }

    public LsidWithExceptionLabelElement createLsidWithExceptionLabelElement(ICdmFormElement parentElement, String labelString,
            LSID initialLsid, int style) {
        LsidWithExceptionLabelElement element = new LsidWithExceptionLabelElement(this, parentElement, labelString, initialLsid, null, style);
        adapt(element);
        parentElement.addElement(element);
        return element;
    }

    public UriWithLabelElement createUriWithLabelElement(ICdmFormElement parentElement, String labelString,
            URI initialUri, int style) {
        UriWithLabelElement element = new UriWithLabelElement(this, parentElement, labelString, initialUri, null, style);
        adapt(element);
        parentElement.addElement(element);
        return element;
    }

    /**
     * @param element
     * @param string
     * @param uri
     * @param style
     * @return
     */
    public OpenUrlSelectorElement createOpenUrlSelectorElement(ICdmFormElement parentElement, String labelString,
            IOpenUrlEnabled openUrlEnabled, int style) {
        OpenUrlSelectorElement element = new OpenUrlSelectorElement(this, parentElement, labelString, openUrlEnabled,
                style);
        adapt(element);
        parentElement.addElement(element);
        return element;
    }

    /**
     *
     * @param parentElement
     * @param labelString
     * @param conversationEnabled
     * @param user
     * @param style
     * @return
     */
    public EditPasswordElement createEditPasswordElement(ICdmFormElement parentElement, String labelString,
            ConversationHolder conversation, User user, int style) {
        EditPasswordElement element = new EditPasswordElement(this, parentElement, labelString, user, conversation);
        adapt(element);
        parentElement.addElement(element);
        return element;
    }

    /**
     * <p>
     * createIntegerTextWithLabelElement
     * </p>
     *
     * @param parentElement
     *            a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement}
     *            object.
     * @param labelString
     *            a {@link java.lang.String} object.
     * @param initialInteger
     *            a {@link java.lang.Integer} object.
     * @param style
     *            a int.
     * @return a
     *         {@link eu.etaxonomy.taxeditor.ui.element.NumberWithLabelElement}
     *         object.
     */
	public NumberWithLabelElement createNumberTextWithLabelElement(
			ICdmFormElement parentElement, String labelString,
			Number initialNumber, int style) {
		NumberWithLabelElement element = new NumberWithLabelElement(this,
				parentElement, labelString, initialNumber, style);
		adapt(element);
		parentElement.addElement(element);
		return element;
	}

    public LanguageStringWithLabelElement createLanguageStringWithLabelElement(ICdmFormElement parentElement,
            String labelString, LanguageString languageString, int style) {
        return createLanguageStringWithLabelElement(parentElement, labelString, languageString, null, false, style);
    }

    public LanguageStringWithLabelElement createLanguageStringWithLabelElement(ICdmFormElement parentElement,
            String labelString, LanguageString languageString, Integer height, boolean isMultiLine, int style) {
        LanguageStringWithLabelElement element = new LanguageStringWithLabelElement(this, parentElement, labelString,
                languageString, height, isMultiLine, style);
        adapt(element);
        parentElement.addElement(element);
        return element;
    }

    /**
     * <p>
     * createKeyValueViewerElement
     * </p>
     *
     * @param parentElement
     *            a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement}
     *            object.
     * @param keyHeading
     *            a {@link java.lang.String} object.
     * @param valueHeading
     *            a {@link java.lang.String} object.
     * @param map
     *            a {@link java.util.Map} object.
     * @return a {@link eu.etaxonomy.taxeditor.ui.element.KeyValueViewerElement}
     *         object.
     */
    public KeyValueViewerElement createKeyValueViewerElement(ICdmFormElement parentElement, String keyHeading,
            String valueHeading, Map<Object, Object> map) {
        KeyValueViewerElement element = new KeyValueViewerElement(this, parentElement, keyHeading, valueHeading, map);
        adapt(element);
        parentElement.addElement(element);
        return element;
    }
    /**
     * @deprecated Use {@link #createDefinedTermComboElement(TermType, ICdmFormElement, String, DefinedTermBase, int)} instead
     */
    @Deprecated
    public <T extends DefinedTermBase> TermComboElement<T> createTermComboElement(Class<T> termComboType,
            ICdmFormElement parentElement, String labelString, T selection, int style) {
        TermComboElement<T> element = new TermComboElement<T>(this, parentElement, termComboType, labelString,
                selection, true,  style);
        adapt(element);
        parentElement.addElement(element);
        return element;
    }

    public <T extends DefinedTermBase> TermComboElement<T> createDefinedTermComboElement(
            TermType termType,
            ICdmFormElement parentElement,
            String labelString,
            T selection,
            int style) {
        TermComboElement<T> element = new TermComboElement<T>(this, parentElement, termType, labelString, selection, true, style);
        adapt(element);
        parentElement.addElement(element);
        return element;
    }

    public <T extends DefinedTermBase> TermComboElement<T> createDefinedTermComboElement(
			TermType termType,
			ICdmFormElement parentElement,
			String labelString,
			T selection,
			boolean addEmptyElement,
			int style) {
		TermComboElement<T> element = new TermComboElement<T>(this, parentElement, termType, labelString, selection, addEmptyElement, style);
		adapt(element);
		parentElement.addElement(element);
		return element;
	}

    public <T extends DefinedTermBase> TermComboElement<T> createDefinedTermComboElement(
            TermVocabulary<?> termVocabulary,
            ICdmFormElement parentElement,
            String labelString,
            T selection,
            int style) {
        TermComboElement<T> element = new TermComboElement<T>(this, parentElement, termVocabulary, labelString, selection, true, style);
        adapt(element);
        parentElement.addElement(element);
        return element;
    }

	public <T extends DefinedTermBase> TermComboElement<T> createDefinedTermComboElement(
	        TermVocabulary<?> termVocabulary,
	        ICdmFormElement parentElement,
	        String labelString,
	        T selection,
	        boolean addEmptyElement,
	        int style) {
	    TermComboElement<T> element = new TermComboElement<T>(this, parentElement, termVocabulary, labelString, selection, addEmptyElement, style);
	    adapt(element);
	    parentElement.addElement(element);
	    return element;
	}

    /**
     * <p>
     * createEnumComboElement
     * </p>
     *
     * @param enumComboType
     *            a
     *            {@link eu.eu.etaxonomy.taxeditor.ui.element.CdmFormFactory.EnumComboType}
     *            object.
     * @param parentElement
     *            a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement}
     *            object.
     * @param style
     *            a int.
     * @return a {@link eu.etaxonomy.taxeditor.ui.term.AbstractEnumComboElement}
     *         object.
     */
	public <T extends IEnumTerm<T>> EnumComboElement<T> createEnumComboElement(
			Class<T> enumComboType, ICdmFormElement parentElement,
			int style) {
        EnumComboElement<T> element = new EnumComboElement<T>(this, parentElement, enumComboType, style);
        adapt(element);
        parentElement.addElement(element);
        return element;
    }

	public <TERM extends DefinedTermBase<TERM>, VOC extends TermVocabulary<TERM>> VocabularyComboElement<TERM, VOC> createVocabularyComboElement(
	        TermType termType, String label, VOC selection, ICdmFormElement parentElement, int style) {
	    VocabularyComboElement<TERM, VOC> element = new VocabularyComboElement<TERM, VOC>(this, parentElement, termType, label, selection, style);
	    adapt(element);
	    parentElement.addElement(element);
	    return element;
	}

    /**
     * <p>
     * createBrowserElement
     * </p>
     *
     * @param imageUri
     *            a {@link java.net.URI} object.
     * @param style
     *            a int.
     * @param parentElement
     *            a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement}
     *            object.
     * @return a {@link eu.etaxonomy.taxeditor.ui.element.BrowserElement}
     *         object.
     */
    public BrowserElement createBrowserElement(ICdmFormElement parentElement, URI imageUri, int style) {
        BrowserElement element = new BrowserElement(this, parentElement, imageUri, style);
        adapt(element);
        parentElement.addElement(element);
        return element;
    }

    /**
     * <p>
     * createImageElement
     * </p>
     *
     * @param parentElement
     *            a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement}
     *            object.
     * @param imageUri
     *            a {@link java.net.URI} object.
     * @param style
     *            a int.
     * @return a {@link eu.etaxonomy.taxeditor.ui.element.ImageElement} object.
     */
    public ImageElement createImageElement(ICdmFormElement parentElement, URI imageUri, int style) {
        ImageElement element = new ImageElement(this, parentElement, imageUri, style);
        adapt(element);
        parentElement.addElement(element);
        return element;
    }

    /**
     * <p>
     * createTextActionElement
     * </p>
     *
     * @param labelString
     *            a {@link java.lang.String} object.
     * @param initialText
     *            a {@link java.lang.String} object.
     * @param style
     *            a int.
     * @param parentElement
     *            a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement}
     *            object.
     * @param buttonLabel
     *            a {@link java.lang.String} object.
     * @return a {@link eu.etaxonomy.taxeditor.ui.element.TextActionElement}
     *         object.
     */
    public TextActionElement createTextActionElement(ICdmFormElement parentElement, String labelString,
            String buttonLabel, String initialText, int style) {
        TextActionElement element = new TextActionElement(this, parentElement, labelString, buttonLabel, initialText,
                style);
        adapt(element);
        parentElement.addElement(element);
        return element;
    }

    /**
     * <p>
     * createCheckbox
     * </p>
     *
     * @param parentElement
     *            a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement}
     *            object.
     * @param label
     *            a {@link java.lang.String} object.
     * @param initialState
     *            a boolean.
     * @param style
     *            a int.
     * @return a {@link eu.etaxonomy.taxeditor.ui.element.CheckboxElement}
     *         object.
     */
    public CheckboxElement createCheckbox(ICdmFormElement parentElement, String label, Boolean initialState, int style) {
        if(initialState==null){
            initialState = Boolean.FALSE;
        }
        CheckboxElement element = new CheckboxElement(this, parentElement, label, initialState, style | orientation);
        adapt(element);
        parentElement.addElement(element);
        return element;
    }

    /**
     * Creates a section as a part of the form.
     *
     * @return the section widget
     * @param section
     *            a
     *            {@link eu.etaxonomy.taxeditor.ui.element.AbstractFormSection}
     *            object.
     */
    public Section adapt(AbstractFormSection section) {
        section.setMenu(section.getLayoutComposite().getMenu());
        adapt(section, true, true);

        // handle focus and property change events for cdm use
        section.addFocusListener(selectionFocusHandler);
        section.setPropertyChangeListeners(propertyChangeListeners);

        if (section.getToggle() != null) {
            section.getToggle().setHoverDecorationColor(getColors().getColor(IFormColors.TB_TOGGLE_HOVER));
            section.getToggle().setDecorationColor(getColors().getColor(IFormColors.TB_TOGGLE));
        }

        section.setFont(boldFontHolder2.getBoldFont(section.getLayoutComposite().getFont()));

        if ((section.getStyle() & ExpandableComposite.TITLE_BAR) != 0
                || (section.getStyle() & ExpandableComposite.SHORT_TITLE_BAR) != 0) {
            getColors().initializeSectionToolBarColors();
            section.setTitleBarBackground(getColors().getColor(IFormColors.TB_BG));
            section.setTitleBarBorderColor(getColors().getColor(IFormColors.TB_BORDER));
        }
        // call setTitleBarForeground regardless as it also sets the label color
        section.setTitleBarForeground(getColors().getColor(IFormColors.TB_TOGGLE));
        return section;
    }

    private class BoldFontHolder2 {
        private Font normalFont;

        private Font boldFont;

        public BoldFontHolder2() {
        }

        public Font getBoldFont(Font font) {
            createBoldFont(font);
            return boldFont;
        }

        private void createBoldFont(Font font) {
            if (normalFont == null || !normalFont.equals(font)) {
                normalFont = font;
                dispose();
            }
            if (boldFont == null) {
                boldFont = FormFonts.getInstance().getBoldFont(getColors().getDisplay(), normalFont);
            }
        }

        public void dispose() {
            if (boldFont != null) {
                FormFonts.getInstance().markFinished(boldFont, getColors().getDisplay());
                boldFont = null;
            }
        }
    }

    /**
     * <p>
     * createToggleableTextField
     * </p>
     *
     * @param parentElement
     *            a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement}
     *            object.
     * @param labelString
     *            a {@link java.lang.String} object.
     * @param initialText
     *            a {@link java.lang.String} object.
     * @param initialState
     *            a boolean.
     * @param style
     *            a int.
     * @return a {@link eu.etaxonomy.taxeditor.ui.element.ToggleableTextElement}
     *         object.
     */
    public ToggleableTextElement createToggleableTextField(ICdmFormElement parentElement, String labelString,
            String initialText, boolean initialState, int style) {
        ToggleableTextElement element = new ToggleableTextElement(this, parentElement, labelString, initialText,
                initialState, style | orientation);
        adapt(element);
        parentElement.addElement(element);
        return element;
    }

    /**
     * <p>
     * createTimePeriodElement
     * </p>
     *
     * @param parentElement
     *            a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement}
     *            object.
     * @param labelString
     *            a {@link java.lang.String} object.
     * @param timePeriod
     *            a {@link eu.etaxonomy.cdm.model.common.TimePeriod} object.
     * @param style
     *            a int.
     * @return a {@link eu.etaxonomy.taxeditor.ui.element.TimePeriodElement}
     *         object.
     */
    public TimePeriodElement createTimePeriodElement(ICdmFormElement parentElement, String labelString,
            TimePeriod timePeriod, int style) {
        TimePeriodElement element = new TimePeriodElement(this, parentElement, labelString, timePeriod, style);
        adapt(element);
        parentElement.addElement(element);
        return element;
    }

	/**
	 * <p>
	 * createGatheringEventUnitElement
	 * </p>
	 *
	 * @param parentElement
	 *            a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement}
	 *            object.
	 * @param labelString
	 *            a {@link java.lang.String} object.
	 * @param timePeriod
	 *            a {@link eu.etaxonomy.cdm.model.common.GatheringEvent} object.
	 * @param style
	 *            a int.
	 * @return a {@link eu.etaxonomy.taxeditor.ui.element.GatheringEventUnitElement}
	 *         object.
	 */
	public GatheringEventUnitElement createGatheringEventUnitElement(
			ICdmFormElement parentElement,
			String labelString,
			DerivedUnitFacade gatheringEvent,
			MinMaxTextSection.UnitType unitType,
			int style) {
		GatheringEventUnitElement element = new GatheringEventUnitElement(this,
				parentElement,
				labelString,
				gatheringEvent,
				unitType,
				style);
		adapt(element);
		parentElement.addElement(element);
		return element;
	}

    /**
     * <p>
     * createPointElement
     * </p>
     *
     * @param style
     *            a int.
     * @param parentElement
     *            a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement}
     *            object.
     * @param point
     *            a {@link eu.etaxonomy.cdm.model.location.Point} object.
     * @return a {@link eu.etaxonomy.taxeditor.ui.element.PointElement} object.
     */
    public PointElement createPointElement(ICdmFormElement parentElement, Point point, int style) {
        PointElement element = new PointElement(this, parentElement, point, style);
        adapt(element);
        parentElement.addElement(element);
        return element;
    }



    /**
     * @param conversationHolder
     * @param parent
     * @param detailsViewer
     * @param i
     * @return
     */
    public EmptySection createEmptySection(CdmFormFactory formFactory, ICdmFormElement parentElement, int style) {
        EmptySection section = new EmptySection(formFactory, parentElement, style);
        parentElement.addElement(section);
        adapt(section);
        return section;
    }

    /**
     * <p>
     * createDateDetailSection
     * </p>
     *
     * @param parentElement
     *            a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement}
     *            object.
     * @param style
     *            a int.
     * @return a {@link eu.etaxonomy.taxeditor.ui.element.DateDetailSection}
     *         object.
     */
    public DateDetailSection createDateDetailSection(ICdmFormElement parentElement, int style) {
        DateDetailSection section = new DateDetailSection(this, parentElement, style);
        parentElement.addElement(section);
        adapt(section);
        return section;
    }

	/**
	 * <p>
	 * createDateDetailSection
	 * </p>
	 *
	 * @param parentElement
	 *            a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement}
	 *            object.
	 * @param style
	 *            a int.
	 * @return a {@link eu.etaxonomy.taxeditor.ui.element.MinMaxTextSection}
	 *         object.
	 */
	public MinMaxTextSection createMinMaxTextSection(ICdmFormElement parentElement, UnitType unitType, int style) {
		MinMaxTextSection section = new MinMaxTextSection(this, parentElement, unitType, style);
		parentElement.addElement(section);
		adapt(section);
		return section;
	}

    /**
     * <p>
     * createPartialElement
     * </p>
     *
     * @param parentElement
     *            a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement}
     *            object.
     * @param labelString
     *            a {@link java.lang.String} object.
     * @param partial
     *            a {@link org.joda.time.Partial} object.
     * @param style
     *            a int.
     * @return a {@link eu.etaxonomy.taxeditor.ui.element.PartialElement}
     *         object.
     */
    public PartialElement createPartialElement(ICdmFormElement parentElement, String labelString, Partial partial,
            int style) {
        PartialElement element = new PartialElement(this, parentElement, labelString, style);
        adapt(element);
        parentElement.addElement(element);
        return element;
    }

    /**
     * <p>
     * addSelectionListener
     * </p>
     *
     * @param listener
     *            a {@link org.eclipse.swt.events.SelectionListener} object.
     */
    public void addSelectionListener(SelectionListener listener) {
        selectionListenerList.add(listener);
    }

    /**
     * <p>
     * removeSelectionListener
     * </p>
     *
     * @param listener
     *            a {@link org.eclipse.swt.events.SelectionListener} object.
     */
    public void removeSelectionListener(SelectionListener listener) {
        if (listener == null) {
            MessagingUtils.error(this.getClass(),
                    "Tried to remove a selection listener from this factories listeners but was null", null);
        } else {
            selectionListenerList.remove(listener);
        }
    }

    /**
     * <p>
     * addPropertyChangeListener
     * </p>
     *
     * @param listener
     *            a {@link org.eclipse.jface.util.IPropertyChangeListener}
     *            object.
     */
    public void addPropertyChangeListener(IPropertyChangeListener listener) {
        if (propertyChangeListeners.contains(listener)) {
            return;
        }
        propertyChangeListeners.add(0, listener);
    }

    /**
     * <p>
     * removePropertyChangeListener
     * </p>
     *
     * @param listener
     *            a {@link org.eclipse.jface.util.IPropertyChangeListener}
     *            object.
     */
    public void removePropertyChangeListener(IPropertyChangeListener listener) {
        propertyChangeListeners.remove(listener);
    }

    /**
     * @return the propertyChangeListeners
     */
    public List<IPropertyChangeListener> getPropertyChangeListeners() {
        return propertyChangeListeners;
    }

    /**
     * <p>
     * createHorizontalSeparator
     * </p>
     *
     * @param parentElement
     *            a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement}
     *            object.
     * @param style
     *            a int.
     * @return a {@link org.eclipse.swt.widgets.Label} object.
     */
    public Label createHorizontalSeparator(ICdmFormElement parentElement, int style) {
        Label separator = this.createSeparator(parentElement.getLayoutComposite(), SWT.HORIZONTAL | style);
        separator.setLayoutData(LayoutConstants.FILL_HORIZONTALLY());
        return separator;
    }

    /**
     * <p>
     * createVersionElement
     * </p>
     *
     * @param parentElement
     *            a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement}
     *            object.
     * @param entity
     *            a {@link eu.etaxonomy.cdm.model.common.VersionableEntity}
     *            object.
     * @param style
     *            a int.
     * @return a
     *         {@link eu.etaxonomy.taxeditor.ui.section.supplemental.VersionElement}
     *         object.
     */
    public VersionElement createVersionElement(ICdmFormElement parentElement, VersionableEntity entity, int style) {
        VersionElement element = new VersionElement(this, parentElement, entity, style);
        adapt(element);
        parentElement.addElement(element);
        return element;
    }

    /**
     * @param cdmBaseSection
     * @param object
     * @param style
     * @return
     */
    public CdmBaseElement createCdmBaseElement(ICdmFormElement parentElement, CdmBase entity, int style) {
        CdmBaseElement element = new CdmBaseElement(this, parentElement, entity, style);
        adapt(element);
        parentElement.addElement(element);
        return element;
    }

    /**
     * <p>
     * createVersionSection
     * </p>
     *
     * @param parentElement
     *            a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement}
     *            object.
     * @param style
     *            a int.
     * @return a
     *         {@link eu.etaxonomy.taxeditor.ui.section.supplemental.VersionSection}
     *         object.
     */
    public VersionSection createVersionSection(ICdmFormElement parentElement, int style) {
        VersionSection section = new VersionSection(this, parentElement, style);
        parentElement.addElement(section);
        adapt(section);
        return section;
    }

    /**
     * @param parent
     * @param i
     * @return
     */
    public CdmBaseSection createCdmBaseSection(ICdmFormElement parentElement, int style) {
        CdmBaseSection section = new CdmBaseSection(this, parentElement, style);
        parentElement.addElement(section);
        adapt(section);
        return section;
    }

    /**
     * <p>
     * createEmptyElement
     * </p>
     *
     * @param parentElement
     *            a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement}
     *            object.
     * @return a {@link eu.etaxonomy.taxeditor.ui.section.EmptyElement} object.
     */
    public EmptyElement createEmptyElement(ICdmFormElement parentElement, String emptyText) {
        EmptyElement element = new EmptyElement(this, parentElement, emptyText, SWT.NULL);
        adapt(element);
        parentElement.addElement(element);
        return element;
    }

    /**
     * <p>
     * createHeadlineSection
     * </p>
     *
     * @param parentElement
     *            a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement}
     *            object.
     * @return a
     *         {@link eu.etaxonomy.taxeditor.ui.section.supplemental.HeadlineSection}
     *         object.
     */
    public HeadlineSection createHeadlineSection(ICdmFormElement parentElement) {
        HeadlineSection section = new HeadlineSection(this, parentElement, SWT.NULL);
        parentElement.addElement(section);
        adapt(section);
        return section;
    }

    /**
     * <p>
     * createParsingMessageElement
     * </p>
     *
     * @param parentElement
     *            a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement}
     *            object.
     * @param parserProblem
     *            a {@link eu.etaxonomy.cdm.strategy.parser.ParserProblem}
     *            object.
     * @param style
     *            a int.
     * @return a
     *         {@link eu.etaxonomy.taxeditor.ui.section.taxon.ParsingMessageElement}
     *         object.
     */
    public ParsingMessageElement createParsingMessageElement(ICdmFormElement parentElement,
            ParserProblem parserProblem, int style) {
        ParsingMessageElement element = new ParsingMessageElement(this, parentElement, parserProblem, style);
        adapt(element);
        parentElement.addElement(element);
        return element;
    }

    public AbstractFormSection createDefinedTermDetailSection(Class definedTermClass, ConversationHolder conversation,
            ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style) {

        AbstractFormSection section = new DefinedTermDetailSection(this, definedTermClass, conversation, parentElement,
                selectionProvider, style);

        parentElement.addElement(section);
        adapt(section);
        return section;

    }

    /**
     * @param definedTermClass
     * @param formElement
     * @param style
     * @return
     */
    public AbstractCdmDetailElement createDefinedTermDetailElement(Class definedTermClass,
            AbstractCdmDetailSection parentElement, int style) {
        AbstractCdmDetailElement element = null;

        if (NamedArea.class.isAssignableFrom(definedTermClass)) {
            element = new eu.etaxonomy.taxeditor.ui.section.vocabulary.NamedAreaDetailElement(this, parentElement);
        } else if (definedTermClass.equals(Feature.class)) {
            element = new FeatureDetailElement(this, parentElement);
        } else {
            element = new DefinedTermDetailElement(this, parentElement);
        }

        adapt(element);
        parentElement.addElement(element);
        return element;
    }


    //--------DetailSections---------
    public NameDetailSection createNameDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        NameDetailSection section = new NameDetailSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public ReferenceDetailSection createReferenceDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        ReferenceDetailSection section = new ReferenceDetailSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public NomenclaturalReferenceDetailSection createNomenclaturalReferenceDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        NomenclaturalReferenceDetailSection section = new NomenclaturalReferenceDetailSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public TaxonBaseDetailSection createTaxonBaseDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        TaxonBaseDetailSection section = new TaxonBaseDetailSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public AuthorshipDetailSection createAuthorshipDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        AuthorshipDetailSection section = new AuthorshipDetailSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public TeamOrPersonBaseDetailSection createTeamOrPersonBaseDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        TeamOrPersonBaseDetailSection section = new TeamOrPersonBaseDetailSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public TeamDetailSection createTeamDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        TeamDetailSection section = new TeamDetailSection(this, conversation, parentElement, null, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public PersonDetailSection createPersonDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        PersonDetailSection section = new PersonDetailSection(this, conversation, parentElement, null, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public DescriptionDetailSection createDescriptionDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        DescriptionDetailSection section = new DescriptionDetailSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public DescriptionElementDetailSection createDescriptionElementDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        DescriptionElementDetailSection section = new DescriptionElementDetailSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public ParsingMessagesSection createParsingMessagesSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        ParsingMessagesSection section = new ParsingMessagesSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public NonViralNameDetailSection createNonViralNameDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        NonViralNameDetailSection section = new NonViralNameDetailSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public MediaDetailsSection createMediaDetailsSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        MediaDetailsSection section = new MediaDetailsSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public FieldUnitDetailSection createFieldUnitDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        FieldUnitDetailSection section = new FieldUnitDetailSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public GatheringEventDetailSection createGatheringEventDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        GatheringEventDetailSection section = new GatheringEventDetailSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public SpecimenHierarchyDetailSection createSpecimenHierarchyDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        SpecimenHierarchyDetailSection section = new SpecimenHierarchyDetailSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public DerivedUnitBaseDetailSection createDerivedUnitBaseDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        DerivedUnitBaseDetailSection section = new DerivedUnitBaseDetailSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public TaxonAssociationDetailSection createTaxonAssociationDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        TaxonAssociationDetailSection section = new TaxonAssociationDetailSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public OriginalLabelDataSection createOriginalLabelDataSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        OriginalLabelDataSection section = new OriginalLabelDataSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public NaturalLanguageSection createNaturalLanguageSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        NaturalLanguageSection section = new NaturalLanguageSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public FeatureDistributionDetailSection createFeatureDistributionDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        FeatureDistributionDetailSection section = new FeatureDistributionDetailSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public ClassificationDetailSection createClassificationDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        ClassificationDetailSection section = new ClassificationDetailSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public TaxonNodeDetailSection createTaxonNodeDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        TaxonNodeDetailSection section = new TaxonNodeDetailSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public PolytomousKeyDetailSection createPolytomousKeyDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        PolytomousKeyDetailSection section = new PolytomousKeyDetailSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public PolytomousKeyNodeDetailSection createPolytomousKeyNodeDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        PolytomousKeyNodeDetailSection section = new PolytomousKeyNodeDetailSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public InstitutionDetailSection createInstitutionDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        InstitutionDetailSection section = new InstitutionDetailSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public FieldUnitGeneralDetailSection createFieldUnitGeneralDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        FieldUnitGeneralDetailSection section = new FieldUnitGeneralDetailSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public DerivedUnitGeneralDetailSection createDerivedUnitGeneralDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        DerivedUnitGeneralDetailSection section = new DerivedUnitGeneralDetailSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public PreservedSpecimenGeneralDetailSection createPreservedSpecimenGeneralDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        PreservedSpecimenGeneralDetailSection section = new PreservedSpecimenGeneralDetailSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public TissueSampleGeneralDetailSection createTissueSampleGeneralDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        TissueSampleGeneralDetailSection section = new TissueSampleGeneralDetailSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public DnaSampleGeneralDetailSection createDnaSampleGeneralDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        DnaSampleGeneralDetailSection section = new DnaSampleGeneralDetailSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public DnaSamplePreparationPreservationSection createDnaSamplePreparationPreservationSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        DnaSamplePreparationPreservationSection section = new DnaSamplePreparationPreservationSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public DnaQualityDetailSection createDnaQualityDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        DnaQualityDetailSection section = new DnaQualityDetailSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public SequenceGeneralDetailSection createSequenceGeneralDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        SequenceGeneralDetailSection section = new SequenceGeneralDetailSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public SequenceContigFileCollectionDetailSection createSequenceContigFileCollectionDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        SequenceContigFileCollectionDetailSection section = new SequenceContigFileCollectionDetailSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public SingleReadPherogramCollectionDetailSection createSingleReadPherogramCollectionDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        SingleReadPherogramCollectionDetailSection section = new SingleReadPherogramCollectionDetailSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public SequenceReferenceCollectionDetailSection createSequenceReferenceCollectionDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        SequenceReferenceCollectionDetailSection section = new SequenceReferenceCollectionDetailSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public SingleReadGeneralDetailSection createSingleReadGeneralDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        SingleReadGeneralDetailSection section = new SingleReadGeneralDetailSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public AmplificationGeneralDetailSection createAmplificationGeneralDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        AmplificationGeneralDetailSection section = new AmplificationGeneralDetailSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public AmplificationPrimerDetailSection createAmplificationPrimerDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        AmplificationPrimerDetailSection section = new AmplificationPrimerDetailSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public AmplificationCloningDetailSection createAmplificationCloningDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        AmplificationCloningDetailSection section = new AmplificationCloningDetailSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public AmplificationGelPhotoCollectionDetailSection createAmplificationGelPhotoCollectionDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        AmplificationGelPhotoCollectionDetailSection section = new AmplificationGelPhotoCollectionDetailSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public MediaSpecimenGeneralDetailSection createMediaSpecimenGeneralDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        MediaSpecimenGeneralDetailSection section = new MediaSpecimenGeneralDetailSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public HybridDetailSection createHybridDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        HybridDetailSection section = new HybridDetailSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public UserDetailSection createUserDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        UserDetailSection section = new UserDetailSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public GroupDetailSection createGroupDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        GroupDetailSection section = new GroupDetailSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public DeterminationDetailSection createDeterminationDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        DeterminationDetailSection section = new DeterminationDetailSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public SampleDesignationDetailSection createSampleDesignationDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        SampleDesignationDetailSection section = new SampleDesignationDetailSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public PreservedSpecimenDeterminationDetailSection createPreservedSpecimenDeterminationDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        PreservedSpecimenDeterminationDetailSection section = new PreservedSpecimenDeterminationDetailSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public TaxonRelationshipDetailSection createTaxonRelationshipDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        TaxonRelationshipDetailSection section = new TaxonRelationshipDetailSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public ReferencedEntityDetailSection createReferencedEntityDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        ReferencedEntityDetailSection section = new ReferencedEntityDetailSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public TermVocabularyDetailSection createTermVocabularyDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        TermVocabularyDetailSection section = new TermVocabularyDetailSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public GatheringEventDetailSection createGatheringSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        GatheringEventDetailSection section = new GatheringEventDetailSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public NamedAreaDetailSection createNamedAreaDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        NamedAreaDetailSection section = new NamedAreaDetailSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public UseRecordDetailSection createUseRecordDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
        UseRecordDetailSection section = new UseRecordDetailSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    private void addAndAdaptSection(ICdmFormElement parentElement, AbstractFormSection<?> section) {
        parentElement.addElement(section);
        adapt(section);
    }

    //--------DetailElements------------

    public UseRecordDetailElement createUseRecordDetailElement(ICdmFormElement parentElement){
        UseRecordDetailElement element = new UseRecordDetailElement(this, parentElement);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public DefinedTermDetailElement<NamedAreaLevel> createNamedAreaLevelElement(ICdmFormElement parentElement){
        DefinedTermDetailElement<NamedAreaLevel> element = new DefinedTermDetailElement<NamedAreaLevel>(this, parentElement);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public eu.etaxonomy.taxeditor.ui.section.vocabulary.NamedAreaDetailElement createNamedAreaDetailElement(ICdmFormElement parentElement){
        eu.etaxonomy.taxeditor.ui.section.vocabulary.NamedAreaDetailElement element = new eu.etaxonomy.taxeditor.ui.section.vocabulary.NamedAreaDetailElement(this, parentElement);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public TermVocabularyDetailElement createTermVocabularyDetailElement(ICdmFormElement parentElement){
        TermVocabularyDetailElement element = new TermVocabularyDetailElement(this, parentElement);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public ReferenceEntityDetailElement createReferenceEntityDetailElement(ICdmFormElement parentElement){
        ReferenceEntityDetailElement element = new ReferenceEntityDetailElement(this, parentElement);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public TaxonRelationshipDetailElement createTaxonRelationshipDetailElement(ICdmFormElement parentElement){
        TaxonRelationshipDetailElement element = new TaxonRelationshipDetailElement(this, parentElement);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public GrantedAuthorityDetailElement createGrantedAuthorityDetailElement(ICdmFormElement parentElement){
        GrantedAuthorityDetailElement element = new GrantedAuthorityDetailElement(this, parentElement);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public GroupDetailElement createGroupDetailElement(ICdmFormElement parentElement){
        GroupDetailElement element = new GroupDetailElement(this, parentElement);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public UserDetailElement createUserDetailElement(ICdmFormElement parentElement){
        UserDetailElement element = new UserDetailElement(this, parentElement);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public HybridDetailElement createHybridDetailElement(ICdmFormElement parentElement){
        HybridDetailElement element = new HybridDetailElement(this, parentElement);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public InstitutionDetailElement createInstitutionDetailElement(ICdmFormElement parentElement){
        InstitutionDetailElement element = new InstitutionDetailElement(this, parentElement);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public PolytomousKeyNodeDetailElement createPolytomousKeyNodeDetailElement(ICdmFormElement parentElement){
        PolytomousKeyNodeDetailElement element = new PolytomousKeyNodeDetailElement(this, parentElement);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public PolytomousKeyDetailElement createPolytomousKeyDetailElement(ICdmFormElement parentElement){
        PolytomousKeyDetailElement element = new PolytomousKeyDetailElement(this, parentElement);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public CollectionDetailElement createCollectionDetailElement(ICdmFormElement parentElement){
        CollectionDetailElement element = new CollectionDetailElement(this, parentElement);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public TaxonNodeDetailElement createTaxonNodeDetailElement(ICdmFormElement parentElement){
        TaxonNodeDetailElement element = new TaxonNodeDetailElement(this, parentElement);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public ClassificationDetailElement createClassificationDetailElement(ICdmFormElement parentElement){
        ClassificationDetailElement element = new ClassificationDetailElement(this, parentElement);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public FeatureDistributionDetailElement createFeatureDistributionDetailElement(ICdmFormElement parentElement){
        FeatureDistributionDetailElement element = new FeatureDistributionDetailElement(this, parentElement);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public NaturalLanguageDetailElement createNaturalLanguageDetailElement(ICdmFormElement parentElement){
        NaturalLanguageDetailElement element = new NaturalLanguageDetailElement(this, parentElement);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public NonViralNameDetailElement createNonViralNameDetailElement(ICdmFormElement parentElement){
        NonViralNameDetailElement element = new NonViralNameDetailElement(this, parentElement);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public DescriptionElementDetailElement createDescriptionElementDetailElement(ICdmFormElement parentElement, int style){
        DescriptionElementDetailElement element = new DescriptionElementDetailElement(this, parentElement, style);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public DescriptionDetailElement createDescriptionDetailElement(ICdmFormElement parentElement, int style){
        DescriptionDetailElement element = new DescriptionDetailElement(this, parentElement, style);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public PersonDetailElement createPersonDetailElement(ICdmFormElement parentElement, int style){
        PersonDetailElement element = new PersonDetailElement(this, parentElement, style);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public TeamDetailElement createTeamDetailElement(ICdmFormElement parentElement, int style){
        TeamDetailElement element = new TeamDetailElement(this, parentElement, style);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public AuthorshipDetailElement createAuthorshipDetailElement(ICdmFormElement parentElement, int style){
        AuthorshipDetailElement element = new AuthorshipDetailElement(this, parentElement, style);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public TaxonBaseDetailElement createTaxonBaseDetailElement(ICdmFormElement parentElement, int style){
        TaxonBaseDetailElement element = new TaxonBaseDetailElement(this, parentElement, style);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public NameDetailElement createNameDetailElement(ICdmFormElement parentElement, int style){
        NameDetailElement element = new NameDetailElement(this, parentElement, style);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public ReferenceDetailElement createReferenceDetailElement(ICdmFormElement parentElement, int style){
        ReferenceDetailElement element = new ReferenceDetailElement(this, parentElement, style);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public NomenclaturalReferenceDetailElement createNomenclaturalReferenceDetailElement(ICdmFormElement parentElement, int style){
        NomenclaturalReferenceDetailElement element = new NomenclaturalReferenceDetailElement(this, parentElement, style);
        addAndAdaptElement(parentElement, element);
        return element;
    }


    public FieldUnitGeneralDetailElement createFieldUnitGeneralDetailElement(ICdmFormElement parentElement){
        FieldUnitGeneralDetailElement element = new FieldUnitGeneralDetailElement(this, parentElement);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public DerivedUnitGeneralDetailElement createDerivedUnitGeneralDetailElement(ICdmFormElement parentElement){
        DerivedUnitGeneralDetailElement element = new DerivedUnitGeneralDetailElement(this, parentElement);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public TaxonAssociationDetailElement createTaxonAssociationDetailElement(ICdmFormElement parentElement){
        TaxonAssociationDetailElement element = new TaxonAssociationDetailElement(this, parentElement);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public PreservedSpecimenGeneralDetailElement createPreservedSpecimenGeneralDetailElement(ICdmFormElement parentElement){
        PreservedSpecimenGeneralDetailElement element = new PreservedSpecimenGeneralDetailElement(this, parentElement);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public TissueSampleGeneralDetailElement createTissueSampleGeneralDetailElement(ICdmFormElement parentElement){
        TissueSampleGeneralDetailElement element = new TissueSampleGeneralDetailElement(this, parentElement);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public DnaSampleGeneralDetailElement createDnaSampleGeneralDetailElement(ICdmFormElement parentElement){
        DnaSampleGeneralDetailElement element = new DnaSampleGeneralDetailElement(this, parentElement);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public DnaSamplePreparationPreservationDetailElement createDnaSamplePreparationPreservationDetailElement(ICdmFormElement parentElement){
        DnaSamplePreparationPreservationDetailElement element = new DnaSamplePreparationPreservationDetailElement(this, parentElement);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public DnaQualityDetailElement createDnaQualityDetailElement(ICdmFormElement parentElement){
        DnaQualityDetailElement element = new DnaQualityDetailElement(this, parentElement);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public SequenceGeneralDetailElement createSequenceGeneralDetailElement(ICdmFormElement parentElement){
        SequenceGeneralDetailElement element = new SequenceGeneralDetailElement(this, parentElement);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public SingleReadGeneralDetailElement createSingleReadGeneralDetailElement(ICdmFormElement parentElement){
        SingleReadGeneralDetailElement element = new SingleReadGeneralDetailElement(this, parentElement);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public PrimerGeneralDetailElement createPrimerGeneralDetailElement(ICdmFormElement parentElement){
        PrimerGeneralDetailElement element = new PrimerGeneralDetailElement(this, parentElement);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public AmplificationGeneralDetailElement createAmplificationGeneralDetailElement(ICdmFormElement parentElement){
        AmplificationGeneralDetailElement element = new AmplificationGeneralDetailElement(this, parentElement);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public AmplificationPrimerDetailElement createAmplificationPrimerDetailElement(ICdmFormElement parentElement){
        AmplificationPrimerDetailElement element = new AmplificationPrimerDetailElement(this, parentElement);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public AmplificationCloningDetailElement createAmplificationCloningDetailElement(ICdmFormElement parentElement){
        AmplificationCloningDetailElement element = new AmplificationCloningDetailElement(this, parentElement);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public AmplificationGelPhotoDetailElement createAmplificationGelPhotoDetailElement(ICdmFormElement parentElement){
        AmplificationGelPhotoDetailElement element = new AmplificationGelPhotoDetailElement(this, parentElement);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public MediaSpecimenGeneralDetailElement createMediaSpecimenGeneralDetailElement(ICdmFormElement parentElement){
        MediaSpecimenGeneralDetailElement element = new MediaSpecimenGeneralDetailElement(this, parentElement);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public MediaDetailElement createMediaDetailElement(ICdmFormElement parentElement){
        MediaDetailElement mediaDetailElement = new MediaDetailElement(this, parentElement);
        addAndAdaptElement(parentElement, mediaDetailElement);
        return mediaDetailElement;
    }

    public GatheringEventDetailElement createGatheringEventDetailElement(ICdmFormElement parentElement) {
        GatheringEventDetailElement element = new GatheringEventDetailElement(this, parentElement);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public FieldUnitDetailElement createFieldUnitDetailElement(ICdmFormElement parentElement) {
        FieldUnitDetailElement element = new FieldUnitDetailElement(this, parentElement);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public SpecimenHierarchyDetailElement createSpecimenHierarchyDetailElement(ICdmFormElement parentElement) {
        SpecimenHierarchyDetailElement element = new SpecimenHierarchyDetailElement(this, parentElement);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public DerivedUnitBaseDetailElement createDerivedUnitBaseDetailElement(ICdmFormElement parentElement) {
        DerivedUnitBaseDetailElement element = new DerivedUnitBaseDetailElement(this, parentElement);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public OriginalLabelDataElement createOriginalLabelDataElement(ICdmFormElement parentElement) {
        OriginalLabelDataElement element = new OriginalLabelDataElement(this, parentElement);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public PreservedSpecimenDetailElement createPreservedSpecimenDetailElement(ICdmFormElement parentElement) {
        PreservedSpecimenDetailElement element = new PreservedSpecimenDetailElement(this, parentElement);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public PreservedSpecimenDeterminationDetailElement createPreservedSpecimenDeterminationDetailElement(ICdmFormElement parentElement) {
        PreservedSpecimenDeterminationDetailElement element = new PreservedSpecimenDeterminationDetailElement(this, parentElement);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public DeterminationDetailElement createDeterminationDetailElement(ICdmFormElement parentElement) {
        DeterminationDetailElement element = new DeterminationDetailElement(this, parentElement);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    public SampleDesignationDetailElement createSampleDesignationDetailElement(ICdmFormElement parentElement) {
        SampleDesignationDetailElement element = new SampleDesignationDetailElement(this, parentElement);
        addAndAdaptElement(parentElement, element);
        return element;
    }

    /**
     * @param parentElement
     * @param element
     */
    private void addAndAdaptElement(ICdmFormElement parentElement, AbstractCdmDetailElement<?> element) {
        adapt(element);
        parentElement.addElement(element);
    }

    //--------EntityCollectionSection----------
    public TeamMemberSection createTeamMemberSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        TeamMemberSection section = new TeamMemberSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public AnnotationSection createAnnotationSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        AnnotationSection section = new AnnotationSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public CreditSection createCreditSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        CreditSection section = new CreditSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public DescriptionElementSourceSection createDescriptionElementSourceSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        DescriptionElementSourceSection section = new DescriptionElementSourceSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public ExtensionSection createExtensionSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        ExtensionSection section = new ExtensionSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public MarkerSection createMarkerSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        MarkerSection section = new MarkerSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public MediaSection createMediaSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        MediaSection section = new MediaSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public DescriptionElementMediaSection createDescriptionElementMediaSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        DescriptionElementMediaSection section = new DescriptionElementMediaSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public MediaRepresentationSection createMediaRepresentationSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        MediaRepresentationSection section = new MediaRepresentationSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public MediaRepresentationPartSection createMediaRepresentationPartSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        MediaRepresentationPartSection section = new MediaRepresentationPartSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public ModifierSection createModifierSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        ModifierSection section = new ModifierSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public NomenclaturalStatusSection createNomenclaturalStatusSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        NomenclaturalStatusSection section = new NomenclaturalStatusSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public NameRelationshipDetailSection createNameRelationshipDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        NameRelationshipDetailSection section = new NameRelationshipDetailSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public SynonymRelationshipDetailSection createSynonymRelationshipDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        SynonymRelationshipDetailSection section = new SynonymRelationshipDetailSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public ProtologueSection createProtologueSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        ProtologueSection section = new ProtologueSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public RightsSection createRightsSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        RightsSection section = new RightsSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public SourceSection createSourceSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        SourceSection section = new SourceSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public ScopeSection createScopeSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        ScopeSection section = new ScopeSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public DescriptionSourceSection createDescriptionSourceSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        DescriptionSourceSection section = new DescriptionSourceSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public TypeDesignationSection createTypeDesignationSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        TypeDesignationSection section = new TypeDesignationSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public StateDataSection createStateDataSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        StateDataSection section = new StateDataSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public StatisticalMeasurementValueSection createStatisticalMeasurementValueSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        StatisticalMeasurementValueSection section = new StatisticalMeasurementValueSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public DescribedSpecimenSection createDescribedSpecimenSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        DescribedSpecimenSection section = new DescribedSpecimenSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public StateVocabularyCollectionSection createStateVocabulariesSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        StateVocabularyCollectionSection section = new StateVocabularyCollectionSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public RecommendedModifierVocabulariesCollectionSection createRecommendedModifierVocabulariesCollectionSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        RecommendedModifierVocabulariesCollectionSection section = new RecommendedModifierVocabulariesCollectionSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public MeasurementUnitCollectionSection createMeasurementUnitCollectionSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        MeasurementUnitCollectionSection section = new MeasurementUnitCollectionSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public StatisticalMeasureCollectionSection createStatisticalMeasureCollectionSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        StatisticalMeasureCollectionSection section = new StatisticalMeasureCollectionSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public CollectingAreasDetailSection createCollectingAreasDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        CollectingAreasDetailSection section = new CollectingAreasDetailSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public GeoScopeDetailSection createGeoScopeDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        GeoScopeDetailSection section = new GeoScopeDetailSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public IdentifierDetailSection createIdentifierDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        IdentifierDetailSection section = new IdentifierDetailSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public CurrentSampleDesignationDetailSection createCurrentSampleDesignationDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        CurrentSampleDesignationDetailSection section = new CurrentSampleDesignationDetailSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public CurrentDeterminationDetailSection createCurrentDeterminationDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        CurrentDeterminationDetailSection section = new CurrentDeterminationDetailSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public PreservedSpecimenCurrentDeterminationDetailSection createPreservedSpecimenCurrentDeterminationDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        PreservedSpecimenCurrentDeterminationDetailSection section = new PreservedSpecimenCurrentDeterminationDetailSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public SampleDesignationHistoryDetailSection createSampleDesignationHistoryDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        SampleDesignationHistoryDetailSection section = new SampleDesignationHistoryDetailSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public DeterminationHistoryDetailSection createDeterminationHistoryDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        DeterminationHistoryDetailSection section = new DeterminationHistoryDetailSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public PreservedSpecimenDeterminationHistoryDetailSection createPreservedSpecimenDeterminationHistoryDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        PreservedSpecimenDeterminationHistoryDetailSection section = new PreservedSpecimenDeterminationHistoryDetailSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public SpecimenCollectionDetailSection createSpecimenCollectionDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        SpecimenCollectionDetailSection section = new SpecimenCollectionDetailSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public PreservedSpecimenSourceCollectionDetailSection createPreservedSpecimenSourceCollectionDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        PreservedSpecimenSourceCollectionDetailSection section = new PreservedSpecimenSourceCollectionDetailSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public SourceCollectionDetailSection createSourceCollectionDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        SourceCollectionDetailSection section = new SourceCollectionDetailSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public ScopeRestrictionSection createScopeRestrictionSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        ScopeRestrictionSection section = new ScopeRestrictionSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public MemberDetailSection createMemberDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        MemberDetailSection section = new MemberDetailSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public GrantedAuthorityDetailSection createGrantedAuthorityDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        GrantedAuthorityDetailSection section = new GrantedAuthorityDetailSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public GroupsByUserDetailSection createGroupsByUserDetailSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        GroupsByUserDetailSection section = new GroupsByUserDetailSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public TaxonomicScopeSection createTaxonomicScopeSection(ConversationHolder conversation, ICdmFormElement parentElement, int style){
        TaxonomicScopeSection section = new TaxonomicScopeSection(this, conversation, parentElement, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

    public AbstractEntityCollectionElement createEntityCollectionElement(AbstractFormSection parentElement,
            Object versionableEntity, SelectionListener removeListener, Color backgroundColor, int style) {
        AbstractEntityCollectionElement element = null;

        Object entity = HibernateProxyHelper.deproxy(versionableEntity);

        if (entity instanceof Annotation) {
            element = new AnnotationElement(this, parentElement, (Annotation) entity, removeListener, style);
        } else if (entity instanceof Person) {
            element = new TeamMemberElement(this, parentElement, (Person) entity, removeListener, style);
        } else if (entity instanceof Credit) {
            element = new CreditElement(this, parentElement, (Credit) entity, removeListener, style);
        } else if (entity instanceof Extension) {
            element = new ExtensionElement(this, parentElement, (Extension) entity, removeListener, style);
        } else if (entity instanceof Marker) {
            element = new MarkerElement(this, parentElement, (Marker) entity, removeListener, style);
        } else if (entity instanceof Media) {
            element = new MediaMetaElement(this, parentElement, (Media) entity, removeListener, true, style);
        } else if (entity instanceof MediaRepresentation) {
            element = new MediaRepresentationElement(this, parentElement, (MediaRepresentation) entity, removeListener,
                    style);
        } else if (entity instanceof ImageFile) {
            element = new ImageFileElement(this, parentElement, (ImageFile) entity, removeListener, style);
        } else if (entity instanceof MediaRepresentationPart) {
            element = new MediaRepresentationPartElement(this, parentElement, (MediaRepresentationPart) entity,
                    removeListener, style);
        } else if (entity instanceof NomenclaturalStatus) {
            element = new NomenclaturalStatusElement(this, parentElement, (NomenclaturalStatus) entity, removeListener,
                    style);
        } else if (entity instanceof Rights) {
            element = new RightsElement(this, parentElement, (Rights) entity, removeListener, style);
        } else if (entity instanceof DescriptionElementSource) {
            element = new DescriptionElementSourceElement(this, parentElement, (DescriptionElementSource) entity,
                    removeListener, style);
        } else if (entity instanceof IdentifiableSource) {
            element = new IdentifiableSourceElement(this, parentElement, (IdentifiableSource) entity, removeListener,
                    style);
        } else if (entity instanceof DefinedTerm) {
            switch(((DefinedTerm)entity).getTermType()) {
            case Scope:
                element = new ScopeElement(this,
                        parentElement,
                        (DefinedTerm) entity,
                        removeListener,
                        style);
                break;
            case Modifier:
                element = new ModifierElement(this,
                        parentElement,
                        (DefinedTerm) entity,
                        removeListener,
                        style);
                break;
            default:
                break;

            }
        } else if (entity instanceof Reference) {
            if(parentElement instanceof SequenceReferenceCollectionDetailSection){
                element = new SequenceReferenceCollectionDetailElement(this, parentElement, (Reference) entity, removeListener, style);
            }
            else{
                element = new DescriptionSourceElement(this, parentElement, (Reference) entity, removeListener, style);
            }
        } else if (entity instanceof NameTypeDesignation) {
            element = new NameTypeDesignationElement(this, parentElement, (NameTypeDesignation) entity, removeListener,
                    style);
        } else if (entity instanceof NameRelationship) {
            element = new NameRelationshipDetailElement(this, parentElement, (NameRelationship) entity, removeListener,
                    style);
        } else if (entity instanceof SynonymRelationship) {
            element = new SynonymRelationshipDetailElement(this, parentElement, (SynonymRelationship) entity, removeListener,
                    style);
        } else if (entity instanceof SpecimenTypeDesignation) {
            element = new SpecimenTypeDesignationElement(this, parentElement, (SpecimenTypeDesignation) entity,
                    removeListener, style);
        } else if (entity instanceof StateData) {
            element = new StateDataElement(this, parentElement, (StateData) entity, removeListener, style);
        } else if (entity instanceof StatisticalMeasurementValue) {
            element = new StatisticalMeasurementValueElement(this, parentElement, (StatisticalMeasurementValue) entity,
                    removeListener, style);
        } else if (entity instanceof DerivedUnit) {
            switch(((DerivedUnit)entity).getRecordBasis()) {
            case LivingSpecimen:
            case PreservedSpecimen:
            case OtherSpecimen:
                element = new SpecimenCollectionDetailElement(this,
                        parentElement,
                        (DerivedUnit) entity,
                        removeListener,
                        style);
                break;
            default:
                element = new DerivedUnitElement(this,
                        parentElement,
                        (DerivedUnit) entity,
                        removeListener,
                        style);
            }

        } else if (entity instanceof NamedArea) {
            element = new NamedAreaDetailElement(this, parentElement, (NamedArea) entity, removeListener, style);
        } else if (entity instanceof DeterminationEvent) {
            element = new DeterminationEventDetailElement(this, parentElement, (DeterminationEvent) entity, removeListener, style);
        } else if (entity instanceof User) {
            element = new MemberDetailElement(this, parentElement, (User) entity, removeListener, style);
        } else if (entity instanceof GrantedAuthority) {
            element = new GrantedAuthorityCollectionElement(this, parentElement, (GrantedAuthorityImpl) entity,
                    removeListener, style);
        } else if (entity instanceof Group) {
            element = new GroupsByUserDetailElement(this, parentElement, (Group) entity, removeListener, style);
        } else if (entity instanceof Taxon) {
            element = new TaxonDetailElement(this, parentElement, (Taxon) entity, removeListener, style);
        } else if (entity instanceof DescriptionElementBase) {
            // this is the special case for protologs, maybe we can do this
            // differently when API improves
            DescriptionElementBase descriptionElement = (DescriptionElementBase) entity;
            if (descriptionElement.getFeature().equals(Feature.PROTOLOGUE())) {
                element = new ProtologueElement(this, parentElement, descriptionElement, removeListener, style);
            }
        } else if (entity instanceof Identifier) {
            if(parentElement instanceof AbstractSampleDesignationDetailSection){
                element = new SampleDesignationTextDetailElement(this, parentElement, (Identifier<DnaSample>) entity, removeListener, backgroundColor, style);
            }
            else{
                element = new IdentifierDetailElement(this, parentElement, (Identifier) entity, removeListener, style);
            }
        } else if (entity instanceof TermVocabulary) {
            TermVocabulary<?> termVocabulary = (TermVocabulary<?>)entity;
            switch (termVocabulary.getTermType()) {
            case State:
                element = new StateVocabularyCollectionElement(this, parentElement, (TermVocabulary<State>) entity, removeListener, backgroundColor, style);
                break;
            case Modifier:
                element = new RecommendedModifierVocabulariesElement(this, parentElement, (TermVocabulary<DefinedTerm>) entity, removeListener, backgroundColor, style);
                break;
            default:
                break;
            }
        } else if (entity instanceof MeasurementUnit) {
            element = new MeasurementUnitCollectionElement(this, parentElement, (MeasurementUnit) entity, removeListener, backgroundColor, style);
        } else if (entity instanceof StatisticalMeasure) {
            element = new StatisticalMeasureCollectionElement(this, parentElement, (StatisticalMeasure) entity, removeListener, backgroundColor, style);
        }

        if (element == null) {
            MessagingUtils.messageDialog("No element for entity", this,
                    "Could not generate element for entity. Looks like the case is not handled already. Check implementation. Entity: "
                            + entity, null);
        }

        else{
            if (backgroundColor != null && !backgroundColor.isDisposed()) {
                element.setPersistentBackground(backgroundColor);
            }
            adapt(element);
            parentElement.addElement(element);
        }

        return element;
    }

    /**
     * <p>
     * Creates a selection element for the given type T.
     * </p>
     * <p>
     * <strong>Selection elements not handled by this method:</strong>
     * <ul>
     * <li>{@link TaxonNodeSelectionElement} see
     * {@link #createTaxonNodeSelectionElement(ConversationHolder, ICdmFormElement, String, TaxonNode, int, int)}
     * </li>
     * <li>{@link NomenclaturalAuthorTeamSelectionElement} see
     * {@link #createNomenclaturalAuthorTeamSelectionElement(ConversationHolder, ICdmFormElement, String, Team, int, int)}
     * </li>
     * </ul>
     * </p>
     *
     * @param clazz
     *            a {@link Class} object of the type that you want the selection
     *            element to handle
     * @param parentElement
     *            a {@link ICdmFormElement} object.
     * @param labelString
     *            a {@link String} object.
     * @param selectionType
     * @param selection
     *            a {@link ICdmBase} object.
     * @param style
     *            a int.
     * @param conversation
     *            a {@link ConversationHolder} object.
     * @return a {@link EntitySelectionElement} object.
     */
    public <T extends ICdmBase> EntitySelectionElement<T> createSelectionElement(Class<T> clazz,
            ConversationHolder conversation, ICdmFormElement parentElement, String labelString, T selection, int mode,
            int style) {
        EntitySelectionElement<T> element = new EntitySelectionElement<T>(this, conversation, parentElement, clazz,
                labelString, selection, mode, style);
        adapt(element);
        parentElement.addElement(element);
        return element;
    }

    /**
     * <p>
     * Creates a selection element for the given type T which shows only the <b>abbreviated</b> title as the label.
     * </p>
     * <p>
     * <strong>Selection elements not handled by this method:</strong>
     * <ul>
     * <li>{@link TaxonNodeSelectionElement} see
     * {@link #createTaxonNodeSelectionElement(ConversationHolder, ICdmFormElement, String, TaxonNode, int, int)}
     * </li>
     * <li>{@link NomenclaturalAuthorTeamSelectionElement} see
     * {@link #createNomenclaturalAuthorTeamSelectionElement(ConversationHolder, ICdmFormElement, String, Team, int, int)}
     * </li>
     * </ul>
     * </p>
     *
     * @param clazz
     *            a {@link Class} object of the type that you want the selection
     *            element to handle
     * @param parentElement
     *            a {@link ICdmFormElement} object.
     * @param labelString
     *            a {@link String} object.
     * @param selectionType
     * @param selection
     *            a {@link ICdmBase} object.
     * @param style
     *            a int.
     * @param conversation
     *            a {@link ConversationHolder} object.
     * @return a {@link EntitySelectionElement} object.
     */
    public <T extends ICdmBase> EntitySelectionElementWithAbbreviatedTitle<T> createSelectionElementWithAbbreviatedTitle(
            Class<T> clazz, ConversationHolder conversation, ICdmFormElement parentElement, String labelString,
            T selection, int mode, int style) {
        EntitySelectionElementWithAbbreviatedTitle<T> element = new EntitySelectionElementWithAbbreviatedTitle<T>(this, conversation, parentElement, clazz,
                labelString, selection, mode, style);
        adapt(element);
        parentElement.addElement(element);
        return element;
    }

    public TaxonNodeSelectionElement createTaxonNodeSelectionElement(ConversationHolder conversation,
            ICdmFormElement parentElement, String labelString, TaxonNode selection, int mode, int style) {
        TaxonNodeSelectionElement element = new TaxonNodeSelectionElement(this, conversation, parentElement,
                labelString, selection, mode, style);
        adapt(element);
        parentElement.addElement(element);
        return element;
    }

    public NomenclaturalAuthorTeamSelectionElement createNomenclaturalAuthorTeamSelectionElement(
            ConversationHolder conversation, ICdmFormElement parentElement, String labelString, Team selection,
            int mode, int style) {
        NomenclaturalAuthorTeamSelectionElement element = new NomenclaturalAuthorTeamSelectionElement(this,
                conversation, parentElement, labelString, selection, mode, style);
        adapt(element);
        parentElement.addElement(element);
        return element;
    }

    /** {@inheritDoc} */
    public LabelElement createLabel(ICdmFormElement parentElement, String text) {
        LabelElement labelElement = new LabelElement(this, parentElement, text);
        adapt(labelElement);
        parentElement.addElement(labelElement);
        return labelElement;
    }

    public DateElement createDateElement(ICdmFormElement formElement, String labelText, DateTime dateTime, int style){
        Label label = new Label(formElement.getLayoutComposite(), style);
        label.setText(labelText+" (yyyy-MM-dd)");
        DateElement dateElement = new DateElement(formElement.getLayoutComposite(), dateTime, style);
        dateElement.initController(this, formElement);
        return dateElement;
    }

    /**
     * <p>
     * Getter for the field <code>selectionProvider</code>.
     * </p>
     *
     * @return a {@link org.eclipse.jface.viewers.ISelectionProvider} object.
     */
    public ISelectionProvider getSelectionProvider() {
        return selectionProvider;
    }

    /**
     * <p>
     * createDetailedDescriptionDetailElement
     * </p>
     *
     * @param parentElement
     *            a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement}
     *            object.
     * @param entity
     *            a
     *            {@link eu.etaxonomy.cdm.model.description.DescriptionElementBase}
     *            object.
     * @param style
     *            a int.
     * @return a
     *         {@link eu.etaxonomy.taxeditor.ui.section.description.detail.AbstractDetailedDescriptionDetailElement}
     *         object.
     */
    public AbstractDetailedDescriptionDetailElement createDetailedDescriptionDetailElement(
            ICdmFormElement parentElement, DescriptionElementBase entity, int style) {
        AbstractDetailedDescriptionDetailElement detailedDescriptionElement = null;

        if (entity instanceof CategoricalData) {
            detailedDescriptionElement = new CategoricalDataDetailElement(this, parentElement,
                    (CategoricalData) entity, style);
        } else if (entity instanceof CommonTaxonName) {
            detailedDescriptionElement = new CommonNameDetailElement(this, parentElement, (CommonTaxonName) entity,
                    style);
        } else if (entity instanceof Distribution) {
            detailedDescriptionElement = new DistributionDetailElement(this, parentElement, (Distribution) entity,
                    style);
        } else if (entity instanceof IndividualsAssociation) {
            detailedDescriptionElement = new IndividualsAssociationDetailElement(this, parentElement,
                    (IndividualsAssociation) entity, style);
        } else if (entity instanceof QuantitativeData) {
            detailedDescriptionElement = new QuantitativeDataDetailElement(this, parentElement,
                    (QuantitativeData) entity, style);
        } else if (entity instanceof TaxonInteraction) {
            detailedDescriptionElement = new TaxonInteractionDetailElement(this, parentElement,
                    (TaxonInteraction) entity, style);
        } else if (entity instanceof TextData) {
            detailedDescriptionElement = new TextDataDetailElement(this, parentElement, (TextData) entity, style);
        } else {
            throw new IllegalStateException("There is no interface for the given description element");
        }
        adapt(detailedDescriptionElement);
        parentElement.addElement(detailedDescriptionElement);
        return detailedDescriptionElement;

    }

    /**
     * Creates a styled text as a part of the form.
     *
     * @param parent
     *            the text parent
     * @param value
     *            the text initial value
     * @param style
     *            the text style
     * @return the text widget
     */
    public StyledText createStyledText(Composite parent, String value, int style) {
        StyledText text = new StyledText(parent, getBorderStyle() | style | getOrientation());
        if (value != null) {
            text.setText(value);
        }
        text.setForeground(getColors().getForeground());
        text.setBackground(getColors().getBackground());
        // text.addFocusListener(visibilityHandler);
        return text;
    }

    public PreservedSpecimenDetailSection createPreservedSpecimenDetailsSection(ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style) {
        PreservedSpecimenDetailSection section = new PreservedSpecimenDetailSection(this, conversation, parentElement, selectionProvider, style);
        addAndAdaptSection(parentElement, section);
        return section;
    }

}
