/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.group.grantedauthority;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseTrackAdapter;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;

import eu.etaxonomy.cdm.model.common.GrantedAuthorityImpl;
import eu.etaxonomy.cdm.persistence.hibernate.permission.CRUD;
import eu.etaxonomy.cdm.persistence.hibernate.permission.CdmAuthority;
import eu.etaxonomy.cdm.persistence.hibernate.permission.Operation;


/**
 * Widget for editing {@link Operation} objects, consisting of 4 checkboxes for CREATE, UPDATE, READ, DELETE operations.
 * 
 * @author cmathew
 * @created Mar 28, 2013
 *
 */
public class CRUDOperationChooser extends Composite {
	
	private Button createCheckButton;
	private Button readCheckButton;
	private Button deleteCheckButton;
	private Button updateCheckButton;
	
	private CdmAuthorityRow caRow;
	private GrantedAuthorityImpl grantedAuthorityI;
	private CdmAuthority cdmAuthority;
	private CdmAuthorityCompositeViewer viewer;
	
	/**
	 * Create the composite.
	 * @param parent
	 * @param style
	 */
	public CRUDOperationChooser(final CdmAuthorityRow parent, int style) {
		super(parent, style);
		setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
		RowLayout rowLayout = new RowLayout(SWT.HORIZONTAL);
		rowLayout.justify = true;
		setLayout(rowLayout);
		
		
		this.caRow = parent;
		createCheckButton = new Button(this, SWT.CHECK);
		createCheckButton.setForeground(Display.getCurrent().getSystemColor(SWT.COLOR_WIDGET_DARK_SHADOW));
		createCheckButton.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseUp(MouseEvent arg0) {
				if(cdmAuthority.getOperation() != null) {
					if(createCheckButton.getSelection()) {
						cdmAuthority.getOperation().add(CRUD.CREATE);
					} else {
						cdmAuthority.getOperation().remove(CRUD.CREATE);
					}
					if(viewer != null) {
						grantedAuthorityI = viewer.updateGrantedAuthority(grantedAuthorityI, cdmAuthority);
						parent.setDirty(true);
					}
					
				}
			}
		});
		createCheckButton.setText("C");
		
		readCheckButton = new Button(this, SWT.CHECK);
		readCheckButton.setEnabled(false);
		readCheckButton.addMouseTrackListener(new MouseTrackAdapter() {
			@Override
			public void mouseHover(MouseEvent arg0) {
			}
		});
		readCheckButton.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseUp(MouseEvent arg0) {
				if(cdmAuthority.getOperation() != null) {
					if(readCheckButton.getSelection()) {
						cdmAuthority.getOperation().add(CRUD.READ);
					} else {
						cdmAuthority.getOperation().remove(CRUD.READ);
					}
					if(viewer != null) {
						grantedAuthorityI = viewer.updateGrantedAuthority(grantedAuthorityI, cdmAuthority);
						parent.setDirty(true);
					}
					
				}
					
			}
		});
		readCheckButton.setText("R");
		
		deleteCheckButton = new Button(this, SWT.CHECK);
		deleteCheckButton.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseUp(MouseEvent arg0) {
				if(cdmAuthority.getOperation() != null) {
					if(deleteCheckButton.getSelection()) {
						cdmAuthority.getOperation().add(CRUD.DELETE);
					} else {
						cdmAuthority.getOperation().remove(CRUD.DELETE);
					}
					if(viewer != null) {
						grantedAuthorityI = viewer.updateGrantedAuthority(grantedAuthorityI, cdmAuthority);
						parent.setDirty(true);
					}
					
				}
			}
		});
		deleteCheckButton.setText("D");
		
		updateCheckButton = new Button(this, SWT.CHECK);
		updateCheckButton.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseUp(MouseEvent arg0) {
				if(cdmAuthority.getOperation() != null) {
					if(updateCheckButton.getSelection()) {
						cdmAuthority.getOperation().add(CRUD.UPDATE);
					} else {
						cdmAuthority.getOperation().remove(CRUD.UPDATE);
					}
					if(viewer != null) {
						grantedAuthorityI = viewer.updateGrantedAuthority(grantedAuthorityI, cdmAuthority);
						parent.setDirty(true);
					}
					
				}
			}
		});
		updateCheckButton.setText("U");
		updateView();
	}

	
	@Override
	protected void checkSubclass() {
		// Disable the check that prevents subclassing of SWT components
	}
	
	/**
	 * Set viewer, CDM Authority and corresponding GrantedAuthorityImpl object
	 * 
	 * @param viewer
	 * @param grantedAuthorityI
	 * @param cdmAuthority
	 */
	public void setAuthority(CdmAuthorityCompositeViewer viewer, 
			GrantedAuthorityImpl grantedAuthorityI,
			CdmAuthority cdmAuthority) {
		this.grantedAuthorityI = grantedAuthorityI;
		this.cdmAuthority = cdmAuthority;
		this.viewer = viewer;
		updateView();
	}
	

	
	/**
	 * Update widget state.
	 */
	private void updateView() {
		createCheckButton.setSelection(false);
		readCheckButton.setSelection(true);
		deleteCheckButton.setSelection(false);
		updateCheckButton.setSelection(false);
		
		
		if(cdmAuthority!= null && cdmAuthority.getOperation() != null) {
			if(cdmAuthority.getOperation().contains(CRUD.CREATE)) {
				createCheckButton.setSelection(true);
			}
			if(cdmAuthority.getOperation().contains(CRUD.READ)) {
				readCheckButton.setSelection(true);
			}
			if(cdmAuthority.getOperation().contains(CRUD.DELETE)) {
				deleteCheckButton.setSelection(true);
			}
			if(cdmAuthority.getOperation().contains(CRUD.UPDATE)) {
				updateCheckButton.setSelection(true);
			}

		}
	}

}
