/**
 *
 */
package eu.etaxonomy.taxeditor.ui.section.media;

import org.eclipse.jface.action.IAction;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Button;

import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.agent.AgentBase;
import eu.etaxonomy.cdm.model.common.LanguageString;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.model.ImageResources;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.dialog.selection.MediaSelectionDialog;
import eu.etaxonomy.taxeditor.ui.element.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.ISelectableElement;
import eu.etaxonomy.taxeditor.ui.element.LanguageStringWithLabelElement;
import eu.etaxonomy.taxeditor.ui.element.SelectionArbitrator;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailSection;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionElement;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;

/**
 * @author n.hoffmann
 * @created Nov 17, 2009
 * @version 1.0
 */
public class MediaMetaElement extends AbstractEntityCollectionElement<Media> implements ISelectableElement, ITogglableMediaElement{

    private EntitySelectionElement<AgentBase> selection_artist;
    private LanguageStringWithLabelElement text_description;
    private LanguageStringWithLabelElement text_title;
    private MediaDetailElement mediaDetailElement;
    private SelectionArbitrator selectionArbitrator;

    public MediaMetaElement(CdmFormFactory cdmFormFactory,
            AbstractFormSection<?> formElement, Media element,
            SelectionListener removeListener, boolean isChoosableEntity, int style) {
        super(cdmFormFactory, formElement, element, removeListener, isChoosableEntity, null, style);
        //make element selectable
        if(cdmFormFactory.getSelectionProvider() != null){
            selectionArbitrator = cdmFormFactory.createSelectionArbitrator(this);
        }
    }

    /** {@inheritDoc} */
    @Override
    public void createControls(ICdmFormElement element, int style) {
        final IAction switchAction = MediaViewSwitch.createSwitch(MediaMetaElement.this);
        Button btnToggleAdvancedMediaView = new Button(getBox(), style);
        btnToggleAdvancedMediaView.setToolTipText(switchAction.getText());
        btnToggleAdvancedMediaView.setImage(ImageResources.getImage(ImageResources.SWITCH_VIEW_TYPE));
        btnToggleAdvancedMediaView.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                switchAction.run();
            }
        });
        text_title = formFactory.createLanguageStringWithLabelElement(element, "Title", null, style);
        selection_artist = formFactory.createSelectionElement(AgentBase.class, getConversationHolder(), element, "Artist", null, EntitySelectionElement.ALL, style);
        text_description = formFactory.createLanguageStringWithLabelElement(element, "Description", null, 100, true, style);
        mediaDetailElement = formFactory.createMediaDetailElement(element);
    }

    /** {@inheritDoc} */
    @Override
    public void setEntity(Media element) {
        this.entity = element;
        mediaDetailElement.setEntity(element);

        selection_artist.setEntity((AgentBase<?>) HibernateProxyHelper
                .deproxy(element.getArtist()));

        text_description.setLanguageString(element.getDescription(CdmStore
                .getDefaultLanguage()));

        LanguageString title = element.getTitle(CdmStore.getDefaultLanguage());

        if (title == null) {
            title = LanguageString.NewInstance("", CdmStore.getDefaultLanguage());
        }

        text_title.setLanguageString(title);
    }

    @Override
    protected Media selectFromDialog() {
        return MediaSelectionDialog.select(AbstractUtility.getShell(), null, null);
    }

    @Override
    public void toggleAdvancedMediaView(){
        mediaDetailElement.toggleAdvancedMediaView();
    }

    /** {@inheritDoc} */
    @Override
    public void handleEvent(Object eventSource) {
        if (eventSource == selection_artist) {
            getEntity().setArtist(selection_artist.getSelection());
        } else if (eventSource == text_description) {
            getEntity().addDescription(text_description.getLanguageString());
        } else if (eventSource == text_title) {
            getEntity().putTitle(text_title.getLanguageString());
            if (getParentElement() instanceof AbstractCdmDetailSection) {
                ((AbstractCdmDetailSection<?>) getParentElement()).updateTitle();
            }
        }
    }

    @Override
    public SelectionArbitrator getSelectionArbitrator() {
        return selectionArbitrator;
    }

}
