// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.section.name;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.ToolBarManager;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Control;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.service.ITaxonNodeService;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.taxeditor.model.ImageResources;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.dialog.selection.NameSelectionDialog;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.CdmPropertyChangeEvent;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailSection;
import eu.etaxonomy.taxeditor.ui.section.ITaxonBaseDetailSection;

/**
 * @author n.hoffmann
 * @created May 20, 2010
 * @version 1.0
 */
public class NonViralNameDetailSection extends AbstractCdmDetailSection<NonViralName>
		implements ITaxonBaseDetailSection {

	private TaxonBase taxonBase;

	/**
	 * <p>Constructor for NonViralNameDetailSection.</p>
	 *
	 * @param formFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 * @param parentElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 * @param selectionProvider a {@link org.eclipse.jface.viewers.ISelectionProvider} object.
	 * @param style a int.
	 */
	public NonViralNameDetailSection(CdmFormFactory formFactory, ConversationHolder conversation,
			ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style) {
		super(formFactory, conversation, parentElement, selectionProvider, style);
	}

	@Override
	protected Control createToolbar() {
		ToolBarManager toolBarManager = new ToolBarManager(SWT.FLAT);

		//choose name
		Action chooseNameAction = new Action("Choose Name", IAction.AS_PUSH_BUTTON){
		    @Override
		    public void run() {
		        TaxonNameBase taxonName = NameSelectionDialog.select(getShell(), getConversationHolder(), null);
		        if(taxonName!=null){
		            if(taxonName.isInstanceOf(NonViralName.class)){
		                CdmStore.getService(ITaxonNodeService.class).list(TaxonNode.class, null, null, null, null);
		                NonViralName nonViralName = HibernateProxyHelper.deproxy(taxonName, NonViralName.class);
		                taxonBase.setName(nonViralName);
		                //				        taxonBase.getTitleCache();
		                setEntity(nonViralName);
		                firePropertyChangeEvent(NonViralNameDetailSection.this);
		            }
		            else{
		                MessagingUtils.warningDialog("Invalid name", this, "The selected name can not be used for this taxon.");
		            }
		        }
		    }
		};
		chooseNameAction.setToolTipText("Choose name for this taxon");
		chooseNameAction.setImageDescriptor(ImageResources.getImageDescriptor(ImageResources.BROWSE_ICON));

		toolBarManager.add(chooseNameAction);

		//clone
		if(getEntity() != null && checkForMultipleNameUsages(getEntity())){

			Action cloneAction = new Action("Clone", IAction.AS_PUSH_BUTTON){
			    @Override
			    public void run() {
					boolean confirm = MessagingUtils.confirmDialog("Confirm cloning", "Do you really want to clone the name?");

					if(confirm){
						NonViralName clonedName;
						clonedName = (NonViralName) getEntity().clone();
						setEntity(clonedName);
						taxonBase.setName(clonedName);
						taxonBase.generateTitle();
						firePropertyChangeEvent(new CdmPropertyChangeEvent(NonViralNameDetailSection.this, null));
					}

				};
			};

			cloneAction.setToolTipText("Clone the name if you do not want to edit the shared instance");

			toolBarManager.add(cloneAction);

		}
		return toolBarManager.createControl(this);
	}


	private boolean checkForMultipleNameUsages(NonViralName nonViralName) {
		return nonViralName.getTaxonBases().size() != 1;
	}

	/** {@inheritDoc} */
	@Override
	public String getHeading() {
		return "Name";
	}

	/** {@inheritDoc} */
	@Override
    public void setTaxonBase(TaxonBase taxon) {
		taxonBase = taxon;
		NonViralName name = (NonViralName) HibernateProxyHelper.deproxy(taxon.getName());
		setEntity(name);
	}

	@Override
	public TaxonBase getTaxonBase() {
		return taxonBase;
	}

	@Override
	protected AbstractCdmDetailElement<NonViralName> createCdmDetailElement(AbstractCdmDetailSection<NonViralName> parentElement, int style) {
	    return formFactory.createNonViralNameDetailElement(parentElement);
	}
}
