// $Id$
/**
 * Copyright (C) 2014 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.ui.section.name;

import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.model.common.TermType;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.SynonymRelationship;
import eu.etaxonomy.cdm.model.taxon.SynonymRelationshipType;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.ui.combo.TermComboElement;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.CheckboxElement;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;
import eu.etaxonomy.taxeditor.ui.element.RootElement;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;

/**
 * @author pplitzner
 * @date 27.01.2014
 *
 */
public class SynonymRelationshipWizardPage extends WizardPage implements IPropertyChangeListener {

    private final CdmFormFactory formFactory;

    private EntitySelectionElement<Taxon> selection_relatedTo;

    private TermComboElement<SynonymRelationshipType> combo_relationshipType;

    private CheckboxElement checkboxProParte;

    private boolean isProParte;

    private CheckboxElement checkboxPartial;

    private boolean isPartial;

    private final Synonym entity;

    private SynonymRelationshipType type;

    private Taxon taxon;

    private RootElement rootElement;

    /**
     * <p>
     * Constructor for NameRelationshipWizardPage.
     * </p>
     *
     * @param callingSection
     *            a
     *            {@link eu.etaxonomy.taxeditor.ui.section.name.NameRelationshipDetailSection}
     *            object.
     */
    protected SynonymRelationshipWizardPage(
            SynonymRelationshipDetailSection callingSection) {
        super("SynonymRelationshipWizardPage");
        setTitle("New Synonym Relationship");
        setDescription(callingSection.getEntity().getTitleCache());
        this.entity = callingSection.getEntity();
        this.formFactory = callingSection.getFormFactory();

        formFactory.addPropertyChangeListener(this);
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets
     * .Composite)
     */
    /** {@inheritDoc} */
    @Override
    public void createControl(Composite parent) {
        this.setPageComplete(false);

        Composite control = formFactory.createComposite(parent);

        control.setLayout(LayoutConstants.LAYOUT(2, false));

        rootElement = new RootElement(formFactory, control);

        combo_relationshipType = formFactory.createDefinedTermComboElement(TermType.SynonymRelationshipType,
                rootElement, "Synonym Relationship Type", null, SWT.NULL);

        selection_relatedTo = formFactory
                .createSelectionElement(Taxon.class,
                        ((SynonymRelationshipWizard) getWizard())
                        .getConversationHolder(), rootElement,
                        "Related to", null, EntitySelectionElement.ALL,
                        SWT.NULL);

        checkboxProParte = formFactory.createCheckbox(rootElement, "Pro Parte Synonym", null, SWT.NULL);

        checkboxPartial = formFactory.createCheckbox(rootElement, "Partial Synonym", null, SWT.NULL);

        setControl(control);
    }

    /**
     * <p>
     * getNameRelationship
     * </p>
     *
     * @return a {@link eu.etaxonomy.cdm.model.name.NameRelationship} object.
     */
    public SynonymRelationship getSynonymRelationship() {
        SynonymRelationship synonymRelationship = taxon.addSynonym(entity, combo_relationshipType.getSelection());
        synonymRelationship.setPartial(isPartial);
        synonymRelationship.setProParte(isProParte);
        return synonymRelationship;
    }

    /** {@inheritDoc} */
    @Override
    public void propertyChange(PropertyChangeEvent event) {
        if (event == null) {
            return;
        }
        Object eventSource = event.getSource();
        if (eventSource == combo_relationshipType) {
            type = combo_relationshipType.getSelection();
        } else if (eventSource == selection_relatedTo) {
            taxon = selection_relatedTo.getEntity();
        } else if (eventSource == checkboxProParte) {
            isProParte = checkboxProParte.getSelection();
        } else if (eventSource == checkboxPartial) {
            isPartial = checkboxPartial.getSelection();
        }

        boolean complete = type != null && taxon != null;
        setPageComplete(complete);
    }

    /** {@inheritDoc} */
    @Override
    public void dispose() {
        rootElement.removeElements();
        formFactory.removePropertyChangeListener(this);
        super.dispose();
    }
}
