// $Id$
/**
* Copyright (C) 2015 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.vocabulary;

import java.util.Collection;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.MeasurementUnit;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.AbstractUnboundEntityCollectionSection;

/**
 * @author pplitzner
 * @date Apr 27, 2015
 *
 */
public class MeasurementUnitCollectionSection extends AbstractUnboundEntityCollectionSection<Feature, MeasurementUnit> {

    public MeasurementUnitCollectionSection(CdmFormFactory formFactory,
            ConversationHolder conversation, ICdmFormElement parentElement, int style) {
        super(formFactory, conversation, parentElement, "Recommended measurement units", style);
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.ui.section.occurrence.dna.AbstractNullEntityCollectionSection#getEntityCollection(java.lang.Object)
     */
    /** {@inheritDoc} */
    @Override
    protected Collection<MeasurementUnit> getEntityCollection(Feature entity) {
        return entity.getRecommendedMeasurementUnits();
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection#createNewElement()
     */
    /** {@inheritDoc} */
    @Override
    public MeasurementUnit createNewElement() {
        return MeasurementUnit.NewInstance();
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.section.AbstractEntityCollectionSection#addElement(eu.etaxonomy.cdm.model.common.IVersionableEntity)
     */
    /** {@inheritDoc} */
    @Override
    public void addElement(MeasurementUnit element) {
        //never gets called
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.section.AbstractEntityCollectionSection#removeElement(eu.etaxonomy.cdm.model.common.IVersionableEntity)
     */
    /** {@inheritDoc} */
    @Override
    public void removeElement(MeasurementUnit element) {
        getEntity().removeRecommendedMeasurementUnit(element);
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.section.AbstractEntityCollectionSection#getEmptyString()
     */
    /** {@inheritDoc} */
    @Override
    public String getEmptyString() {
        return "No recommended measurement units yet.";
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.section.AbstractEntityCollectionSection#getTooltipString()
     */
    /** {@inheritDoc} */
    @Override
    protected String getTooltipString() {
        return "Add a measurement unit";
    }

}
