// $Id$
/**
* Copyright (C) 2015 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.view;

import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;

import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.store.internal.TaxeditorStorePlugin;

/**
 * @author pplitzner
 * @date Jul 7, 2015
 *
 */
public class CdmViewerUtil {

    public static  Map<Entry<Class<?>, String>, ICdmViewer> getNameViewerMap(Object input){
        Map<Entry<Class<?>, String>, ICdmViewer> nameViewerMap = new HashMap<Entry<Class<?>, String>, ICdmViewer>();
    
        IExtensionRegistry reg = Platform.getExtensionRegistry();
        IConfigurationElement[] extensions = reg
                .getConfigurationElementsFor("eu.etaxonomy.taxeditor.store.cdmViewer");
        for (IConfigurationElement configElement : extensions) {
            try {
                Object object = configElement.createExecutableExtension("class");
                if(object instanceof ICdmViewer){
                    ICdmViewer cdmViewer = (ICdmViewer)object;
                    Map<Class<?>, String> viewerClasses = cdmViewer.getViewerClasses(input);
                    for (Entry<Class<?>, String> entry : viewerClasses.entrySet()) {
                        nameViewerMap.put(entry, cdmViewer);
                    }
                }
                else{
                    MessagingUtils.error(CdmViewerChooser.class, new Status(IStatus.ERROR, TaxeditorStorePlugin.PLUGIN_ID, "Could not load cdmViewer extension"));
                }
            } catch (CoreException e) {
                MessagingUtils.error(CdmViewerChooser.class, "Could not load cdmViewer extension", e);
            }
        }
        return nameViewerMap;
    }

}
