// $Id$
/**
* Copyright (C) 2015 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.view;

import java.util.Map;


/**
 * Implementors of this interface provide a mapping of input elements to views
 * or editors which can display information or provide editing functionality for
 * the input elements.
 *
 * @author pplitzner
 * @date Feb 23, 2015
 *
 */
public interface ICdmViewer {

    /**
     * For the given input a map is returned specifying the available viewer
     * classes as keys and their string representation as values
     *
     * @param input
     *            the input for which the viewer classes should be returned
     * @return a map holding the viewer classes as keys and their string
     *         representations as values
     */
    public Map<Class<?>, String> getViewerClasses(Object input);

    /**
     * Opens the viewer defined by the given viewerClass for the given input.
     * @param input the input for which a viewer should be opened
     * @param viewerClass the qualified class name of the viewer
     */
    public void show(Object input, Class<?> viewerClass);

}
