// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.view.detail;

import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreeNode;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchPart;

import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.model.FeatureNodeContainer;
import eu.etaxonomy.taxeditor.model.IPartContentHasDetails;
import eu.etaxonomy.taxeditor.model.IPartContentHasSupplementalData;
import eu.etaxonomy.taxeditor.view.AbstractCdmEditorViewPart;

/**
 * <p>DetailsViewPart class.</p>
 *
 * @author n.hoffmann
 * @created Jun 10, 2010
 * @version 1.0
 */
public class DetailsViewPart extends AbstractCdmEditorViewPart implements IPartContentHasSupplementalData{

	/** Constant <code>ID="eu.etaxonomy.taxeditor.editor.forms.det"{trunked}</code> */
	public static String ID = "eu.etaxonomy.taxeditor.view.detail";

	private DetailsViewer viewer;

	/** {@inheritDoc} */
	@Override
	public void createViewer(Composite parent) {

		viewer = new DetailsViewer(parent, this);
		getSite().setSelectionProvider(viewer);
	}

	@Override
    protected void selectionChanged_internal(IWorkbenchPart part, ISelection selection){
	    if(AbstractUtility.getActiveEditor() == null){
            showEmptyPage();
            return;
        }

        if(part == this){
            return;
        }

        if(!(selection instanceof IStructuredSelection)){
            return;
        }

        IStructuredSelection structuredSelection = (IStructuredSelection) selection;

        if((part instanceof IEditorPart) || (part instanceof IPartContentHasDetails)) {
            if(structuredSelection.size() != 1){
                setPartName(createPartTitle(null));
                showEmptyPage();
                return;
            }

            // do not show details for feature nodes TODO really?
            if(structuredSelection.getFirstElement() instanceof FeatureNodeContainer){
                // do show the map for distributions
                Feature feature = ((FeatureNodeContainer) ((IStructuredSelection) selection).getFirstElement()).getFeature();
                if(!feature.equals(Feature.DISTRIBUTION())){
                    setPartName(createPartTitle(null));
                    showEmptyPage();
                    return;
                }
            }


            setPartName(createPartTitle(((IStructuredSelection) selection).getFirstElement()));
            showViewer(part, structuredSelection);
        }else{
            setPartName(createPartTitle(null));
            showEmptyPage();
        }
	}

	private String getViewName(){
	    return "Details";
	}

	private String createPartTitle(Object selectedObject){
	    if(selectedObject!=null){
	        if(selectedObject instanceof TreeNode){
	            selectedObject = ((TreeNode) selectedObject).getValue();
	        }
	        if(selectedObject instanceof SpecimenOrObservationBase){
	            return getViewName()+": "+HibernateProxyHelper.deproxy(selectedObject, SpecimenOrObservationBase.class).getRecordBasis();
	        }
	        if(selectedObject instanceof CdmBase){
	            return getViewName()+": "+HibernateProxyHelper.deproxy(selectedObject, CdmBase.class).getClass().getSimpleName();
	        }
	        return getViewName()+": "+selectedObject.getClass().getSimpleName();
	    }
	    return getViewName();
	}

	@Override
	public void showEmptyPage() {
	    viewer.setSelection(null);
	    super.showEmptyPage();
	}

	/** {@inheritDoc} */
	@Override
	public boolean postOperation(CdmBase objectAffectedByOperation) {

		viewer.setInput(objectAffectedByOperation);

		return super.postOperation(objectAffectedByOperation);
	}


	/** {@inheritDoc} */
	@Override
	public Viewer getViewer() {
		return viewer;
	}


	/** {@inheritDoc} */
	@Override
	public void dispose() {
		selectionService.removePostSelectionListener(this);
		super.dispose();

	}

	@Override
    public boolean onComplete() {
		return true;
	}
}
