// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.view.supplementaldata;

import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.forms.widgets.ExpandableComposite;

import eu.etaxonomy.cdm.model.common.AnnotatableEntity;
import eu.etaxonomy.cdm.model.common.IdentifiableEntity;
import eu.etaxonomy.cdm.model.common.VersionableEntity;
import eu.etaxonomy.cdm.model.media.IdentifiableMediaEntity;
import eu.etaxonomy.taxeditor.model.IElementHasDetails;
import eu.etaxonomy.taxeditor.preference.IPreferenceKeys;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.RootElement;
import eu.etaxonomy.taxeditor.ui.section.media.MediaSection;
import eu.etaxonomy.taxeditor.ui.section.supplemental.AnnotationSection;
import eu.etaxonomy.taxeditor.ui.section.supplemental.CdmBaseSection;
import eu.etaxonomy.taxeditor.ui.section.supplemental.CreditSection;
import eu.etaxonomy.taxeditor.ui.section.supplemental.ExtensionSection;
import eu.etaxonomy.taxeditor.ui.section.supplemental.HeadlineSection;
import eu.etaxonomy.taxeditor.ui.section.supplemental.MarkerSection;
import eu.etaxonomy.taxeditor.ui.section.supplemental.RightsSection;
import eu.etaxonomy.taxeditor.ui.section.supplemental.SourceSection;
import eu.etaxonomy.taxeditor.ui.section.supplemental.VersionSection;
import eu.etaxonomy.taxeditor.view.AbstractCdmDataViewer;
import eu.etaxonomy.taxeditor.view.AbstractCdmViewPart;
import eu.etaxonomy.taxeditor.view.detail.DetailsViewer;

/**
 * <p>SupplementalDataViewer class.</p>
 *
 * @author n.hoffmann
 * @created Feb 12, 2010
 * @version 1.0
 */
public class SupplementalDataViewer extends AbstractCdmDataViewer implements ISelectionChangedListener {

	private boolean showDebug;


	/**
	 * <p>Constructor for SupplementalDataViewer.</p>
	 *
	 * @param parent a {@link org.eclipse.swt.widgets.Composite} object.
	 * @param viewPart a {@link eu.etaxonomy.taxeditor.view.AbstractCdmViewPart} object.
	 */
	public SupplementalDataViewer(Composite parent, AbstractCdmViewPart viewPart) {
		super(parent, viewPart);
	}


	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.Viewer#getSelection()
	 */
	/** {@inheritDoc} */
	@Override
	public ISelection getSelection() {
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.Viewer#setSelection(org.eclipse.jface.viewers.ISelection, boolean)
	 */
	/** {@inheritDoc} */
	@Override
	public void setSelection(ISelection selection, boolean reveal) {

	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.view.AbstractDetailsViewer#showParts(eu.etaxonomy.cdm.model.common.AnnotatableEntity)
	 */
	/** {@inheritDoc} */
	@Override
	protected void showParts() {

		showDebug = PreferencesUtil.getPreferenceStore().getBoolean(IPreferenceKeys.SHOW_DEBUG_INFORMATION);

		Object input;

		if(getInput() instanceof IElementHasDetails){
			input = ((IElementHasDetails) getInput()).getData();
		}else{
			input = getInput();
		}

		destroySections();
		//1. Headline
		createHeadlineSection(rootElement);
		//2. Annotatable entitiy
		if(input instanceof AnnotatableEntity){
				createAnnotationSections(rootElement);
		}
		//3. Identifiable entity
		if(input instanceof IdentifiableEntity){
			createIdentifiableSections(rootElement);
		}
		//4. Identifiable media entity
		if(input instanceof IdentifiableMediaEntity){
			createIdentifiableMediaSections(rootElement);
		}
		//5. Versionable
		if(input instanceof VersionableEntity){
				createVersionSection(rootElement);
		}
		//6. CdmBase
		if(showDebug){
			createCdmBaseSection(rootElement);
		}

		layout();
	}



	private void createHeadlineSection(RootElement parent){
		HeadlineSection headlineSection = formFactory.createHeadlineSection(parent);
		addPart(headlineSection);
	}

    private void createAnnotationSections(RootElement parent) {
        AnnotationSection annotationSection = formFactory.createAnnotationSection(getConversationHolder(), parent, ExpandableComposite.TWISTIE);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        MarkerSection markerSection = formFactory.createMarkerSection(getConversationHolder(), parent, ExpandableComposite.TWISTIE);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        addPart(annotationSection);
        addPart(markerSection);
    }

    private void createIdentifiableSections(RootElement parent) {
        CreditSection creditSection = formFactory.createCreditSection(getConversationHolder(), parent, ExpandableComposite.TWISTIE);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        ExtensionSection extensionSection = formFactory.createExtensionSection(getConversationHolder(), parent, ExpandableComposite.TWISTIE);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        RightsSection rightsSection = formFactory.createRightsSection(getConversationHolder(), parent, ExpandableComposite.TWISTIE);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        SourceSection sourceSection = formFactory.createSourceSection(getConversationHolder(), parent, ExpandableComposite.TWISTIE);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        addPart(creditSection);
        addPart(extensionSection);
        addPart(rightsSection);
        addPart(sourceSection);

	}

    private void createIdentifiableMediaSections(RootElement parent) {
        MediaSection mediaSection = formFactory.createMediaSection(getConversationHolder(), parent, ExpandableComposite.TWISTIE);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        addPart(mediaSection);
    }

	private void createVersionSection(RootElement parent){
		VersionSection versionSection = formFactory.createVersionSection(parent, ExpandableComposite.NO_TITLE | ExpandableComposite.EXPANDED);
		addPart(versionSection);
	}

	/**
	 * @param rootElement
	 */
	private void createCdmBaseSection(RootElement parent) {

		CdmBaseSection cdmBaseSection = formFactory.createCdmBaseSection(parent, ExpandableComposite.NO_TITLE | ExpandableComposite.EXPANDED);
		addPart(cdmBaseSection);
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ISelectionChangedListener#selectionChanged(org.eclipse.jface.viewers.SelectionChangedEvent)
	 */
	/** {@inheritDoc} */
	@Override
    public void selectionChanged(SelectionChangedEvent event) {
		if(event.getSource() instanceof DetailsViewer){
			if(event.getSelection() == CdmFormFactory.EMPTY_SELECTION){
				setInput(null);
				return;
			}

			IStructuredSelection selection = (IStructuredSelection) event.getSelection();
			setInput(selection.getFirstElement());
		}
	}
}
