// $Id$
/**
 * Copyright (C) 2014 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.ui.section.occurrence.dna;

import eu.etaxonomy.cdm.model.molecular.Amplification;
import eu.etaxonomy.cdm.model.molecular.AmplificationResult;
import eu.etaxonomy.cdm.model.molecular.Primer;
import eu.etaxonomy.cdm.model.molecular.SingleRead;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;
import eu.etaxonomy.taxeditor.ui.section.supplemental.AnnotationSection;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;

/**
 * @author pplitzner
 * @date 06.01.2014
 *
 */
public class SingleReadGeneralDetailElement extends AbstractCdmDetailElement<SingleRead> {

    private EntitySelectionElement<Primer> selectionPrimer;
    private EntitySelectionElement<Amplification> selectionAmplification;

    /**
     * @param formFactory
     * @param formElement
     */
    public SingleReadGeneralDetailElement(CdmFormFactory formFactory, ICdmFormElement formElement) {
        super(formFactory, formElement);
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * eu.etaxonomy.taxeditor.section.AbstractCdmDetailElement#createControls
     * (eu.etaxonomy.taxeditor.forms.ICdmFormElement, java.lang.Object, int)
     */
    @Override
    protected void createControls(ICdmFormElement formElement, SingleRead entity, int style) {
        selectionPrimer = formFactory.createSelectionElement(Primer.class, getConversationHolder(), formElement, "Primer", entity.getPrimer(), EntitySelectionElement.ALL, style);
        AmplificationResult amplificationResult = entity.getAmplificationResult();
        if(amplificationResult==null){
            amplificationResult = AmplificationResult.NewInstance();
            amplificationResult.addSingleRead(entity);
        }
        selectionAmplification = formFactory.createSelectionElement(Amplification.class, getConversationHolder(), formElement, "Amplification", amplificationResult.getAmplification(), EntitySelectionElement.ALL, style);

        AnnotationSection annotationSection = formFactory.createAnnotationSection(getConversationHolder(), formElement, style);
        annotationSection.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
        annotationSection.setEntity(entity);

        AmplificationCloningDetailSection cloningSection = formFactory.createAmplificationCloningDetailSection(getConversationHolder(), formElement, formFactory.getSelectionProvider(), style);
        cloningSection.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
        cloningSection.setEntity(amplificationResult);

        AmplificationGelPhotoCollectionDetailSection gelPhotoSection = formFactory.createAmplificationGelPhotoCollectionDetailSection(getConversationHolder(), formElement, style);
        gelPhotoSection.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
        gelPhotoSection.setEntity(amplificationResult);
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * eu.etaxonomy.taxeditor.section.AbstractCdmDetailElement#handleEvent(java
     * .lang.Object)
     */
    @Override
    public void handleEvent(Object eventSource) {
        if(eventSource==selectionPrimer){
            getEntity().setPrimer(selectionPrimer.getSelection());
        }
        else {
            if(eventSource==selectionAmplification){
                AmplificationResult amplificationResult = getEntity().getAmplificationResult();
                if(amplificationResult==null){
                    amplificationResult = AmplificationResult.NewInstance();
                }
                amplificationResult.setAmplification(selectionAmplification.getSelection());
                amplificationResult.addSingleRead(getEntity());
            }
        }
    }
}
