// $Id$
/**
* Copyright (C) 2013 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.occurrence.dna;

import org.apache.log4j.Logger;

import eu.etaxonomy.cdm.api.service.IVocabularyService;
import eu.etaxonomy.cdm.model.agent.AgentBase;
import eu.etaxonomy.cdm.model.agent.Institution;
import eu.etaxonomy.cdm.model.common.DefinedTerm;
import eu.etaxonomy.cdm.model.common.TermType;
import eu.etaxonomy.cdm.model.common.TermVocabulary;
import eu.etaxonomy.cdm.model.common.TimePeriod;
import eu.etaxonomy.cdm.model.common.VocabularyEnum;
import eu.etaxonomy.cdm.model.molecular.DnaSample;
import eu.etaxonomy.cdm.model.occurrence.Collection;
import eu.etaxonomy.cdm.model.occurrence.PreservationMethod;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.combo.TermComboElement;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.CheckboxElement;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement;
import eu.etaxonomy.taxeditor.ui.element.TimePeriodElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;

/**
 * @author pplitzner
 * @date 16.12.2013
 *
 */
public class TissueSampleGeneralDetailElement extends AbstractCdmDetailElement<DnaSample> {

    private final TermVocabulary plantKindOfUnitVocabulary = CdmStore.getService(IVocabularyService.class).find(VocabularyEnum.PlantKindOfUnit.getUuid());

    static private final Logger logger = Logger.getLogger(TissueSampleGeneralDetailElement.class);

    private TermComboElement<DefinedTerm> comboKindOfTissue;
    private EntitySelectionElement<Institution> selectionInstitution;
    private EntitySelectionElement<AgentBase> selectionStaff;
    private TimePeriodElement datePreparation;
    private TermComboElement<DefinedTerm> comboPreservationMedium;
    private EntitySelectionElement<Collection> selectionCollection;
    private TextWithLabelElement textAccessionNumber;
    private TextWithLabelElement textBarcode;
    private CheckboxElement checkIsPublish;


    /**
     * @param formFactory
     * @param formElement
     */
    public TissueSampleGeneralDetailElement(CdmFormFactory formFactory, ICdmFormElement formElement) {
        super(formFactory, formElement);
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * eu.etaxonomy.taxeditor.section.AbstractCdmDetailElement#createControls
     * (eu.etaxonomy.taxeditor.forms.ICdmFormElement, java.lang.Object, int)
     */
    @Override
    protected void createControls(ICdmFormElement formElement, DnaSample entity, int style) {
        comboKindOfTissue = formFactory.createDefinedTermComboElement(plantKindOfUnitVocabulary, formElement, "Kind Of Tissue", entity.getKindOfUnit(), style);
        TimePeriod preparationDate = null;
        DefinedTerm preservationMedium = null;
        AgentBase<?> staff = null;
        Institution institution = null;
        if(entity.getDerivedFrom()!=null){
            staff = entity.getDerivedFrom().getActor();
            preparationDate = entity.getDerivedFrom().getTimeperiod();
            institution = entity.getDerivedFrom().getInstitution();
        }
        if(entity.getPreservation()!=null){
            preservationMedium = entity.getPreservation().getMedium();
        }
        selectionInstitution = formFactory.createSelectionElement(Institution.class, getConversationHolder(), formElement, "Institution", institution, EntitySelectionElement.ALL, style);
        selectionStaff = formFactory.createSelectionElement(AgentBase.class, getConversationHolder(), formElement, "Staff", staff, EntitySelectionElement.ALL,style);
        datePreparation = formFactory.createTimePeriodElement(formElement, "Preparation date", preparationDate, style);
        comboPreservationMedium = formFactory.createDefinedTermComboElement(TermType.MaterialOrMethod, formElement, "Preservation medium", preservationMedium, style);
        selectionCollection = formFactory.createSelectionElement(Collection.class, getConversationHolder(), formElement, "Collection", entity.getCollection(), EntitySelectionElement.ALL, style);
        textAccessionNumber = formFactory.createTextWithLabelElement(formElement, "Accession Number", entity.getAccessionNumber(), style);
        textBarcode = formFactory.createTextWithLabelElement(formElement, "Barcode", entity.getBarcode(), style);
        checkIsPublish = formFactory.createCheckbox(formElement, "Publish", entity.isPublish(), style);

    }

    /*
     * (non-Javadoc)
     *
     * @see
     * eu.etaxonomy.taxeditor.section.AbstractCdmDetailElement#handleEvent(java
     * .lang.Object)
     */
    @Override
    public void handleEvent(Object eventSource) {
        if(eventSource==comboKindOfTissue) {
            getEntity().setKindOfUnit(comboKindOfTissue.getSelection());
        }
        else if(eventSource==comboPreservationMedium) {
            addPreservationMethod().setMedium(comboPreservationMedium.getSelection());
        }
        else if(eventSource==selectionCollection){
            getEntity().setCollection(selectionCollection.getSelection());
        }
        else if(eventSource==textAccessionNumber){
            getEntity().setAccessionNumber(textAccessionNumber.getText());
        }
        else if(eventSource==textBarcode){
            getEntity().setBarcode(textBarcode.getText());
        }
        else if(eventSource==datePreparation){
            if(getEntity().getDerivedFrom()!=null){
                getEntity().getDerivedFrom().setTimeperiod(datePreparation.getTimePeriod());
            }
        }
        else if(eventSource==selectionInstitution){
            if(getEntity().getDerivedFrom()!=null){
                getEntity().getDerivedFrom().setInstitution(selectionInstitution.getSelection());
            }
        }
        else if(eventSource==selectionStaff){
            if(getEntity().getDerivedFrom()!=null){
                getEntity().getDerivedFrom().setActor(selectionStaff.getSelection());
            }
        }
        else if(eventSource==checkIsPublish){
            getEntity().setPublish(checkIsPublish.getSelection());
        }
    }

    private PreservationMethod addPreservationMethod(){
        PreservationMethod preservationMethod = null;
        preservationMethod = getEntity().getPreservation();
        if(getEntity().getPreservation()==null){
            preservationMethod = PreservationMethod.NewInstance();
            getEntity().setPreservation(preservationMethod);
        }
        return preservationMethod;
    }
}
